//! \file    Order.cpp
//! \brief   Declaration and implementation of the Order class, which
//!          handles order of computation between modules, spaces and trajectories.
//! \version 2008/07/24 (yyyy/mm/dd) 
//! \author  Luigi Nardi <luiginardi(at)gmail.com> 
//
// This file is part of YAO, a framework for variational assimilation in
// numerical models.
// Copyright (c) 2001-onwards LOCEAN. All rights reserved.
//
// This program may be redistributed and/or modified under the terms of the GNU
// General Public License as published by the Free Software Foundation, either
// version 2 of the license, or (at your option) any later version.
//
// This program is provided AS IS and WITHOUT ANY WARRANTY, including any
// implied warranty of DESIGN, MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE.

#include "Order.hpp"
//#include "../help/Helper.hpp"

using namespace std;
using namespace yao;

/* 
 *	Translate the value in input in a string and then returns it.
 *	The integer value in input is one of these possibles cases:
 *	YA1 = 1 ; YB1 = -1 ; YA2 = 2 ; YB2 = -2 ; YA3 = 3 ; YB3 = -3 ;
 *	@param anAxe the integer token to be firstly translated.
 *	@return the string that represents the axe.
 */
string Order::translateAxe(int anAxe){ 
  string token;
  switch(anAxe){
    case 1:
      token = "YA1";
      break;
    case -1:
      token = "YB1";
      break;
    case 2:
      token = "YA2";
      break;
    case -2:
      token = "YB2";
      break;
    case 3:
      token = "YA3";
      break;
    case -3:
      token = "YB3";
      break;
  }
  return token;
} 

//-----------------------------------------------------------------------------------
// The part that follow allow to reverse the vector of tokens (forward <-> backward)

/*
 * Put in the opposite order the vector of tokens of a directive order modinspace or spaceintraj.
 * i.e.: we have an order like this: 
 * order modinspace space
 *	order YA1 YA2
 *	  A B C
 *	forder
 * forder.
 * The vector will contain: order, YA1, YA2, A, B, C, forder.
 * The opposite vector will contain: order, YB1, YB2, C, B, A, forder. 
 * This is done in order to generate the code for the backward function. We could reverse a lot of time the vector of tokens and 
 * each time we obtain the precedent configuration.
 */
void Order::backOrder(){
  int wlevel = 0;
  struct st_token *atoken, *ptoken;
  // We build a tree with the initial order of the table of tokens orderTokens
  rootToken = atoken = ptoken = NULL;

  // Creation of the root (this will allow to manage more than one order in the same directive modinspace or spaceintraj) 
  rootToken = (struct st_token*) malloc(sizeof(struct st_token));
  ENFORCE(rootToken)("order backOrder() root: error on malloc \n");
  sprintf(rootToken->token, "Yroot");
  rootToken->level = wlevel;
  rootToken->brother = NULL;
  rootToken->father = NULL;
  rootToken->son = NULL;
  ptoken = rootToken;

  for(vector<string>::iterator tok=orderTokens.begin(); tok<orderTokens.end(); tok++){
    // When we check that we are managing an axe we reverse this axe
    if      (*tok=="YA1") *tok="YB1"; 
    else if (*tok=="YA2") *tok="YB2";
    else if (*tok=="YA3") *tok="YB3";
    else if (*tok=="YB1") *tok="YA1";
    else if (*tok=="YB2") *tok="YA2";
    else if (*tok=="YB3") *tok="YA3";

    // The construction of the tree begin here: we allocate and then we initialize
    atoken = (struct st_token*) malloc(sizeof(struct st_token));
    ENFORCE(atoken)("order bakorder() token: error on malloc \n");

    sprintf(atoken->token, ((string)(*tok)).c_str());
    atoken->brother = NULL;
    atoken->father = NULL;
    atoken->son = NULL;

    if(strcmp(ptoken->token,"order")==0 || strcmp(ptoken->token,"ORDER")==0 || ptoken==rootToken){
      ++wlevel;
      ptoken->son = atoken;
      atoken->father = ptoken;
      atoken->level  = wlevel;
      ptoken->brother = NULL;
    }
    else if((strcmp(ptoken->token,"forder")==0 || strcmp(ptoken->token,"FORDER")==0) && (ptoken->father!=rootToken)){
      --wlevel;
      ptoken->father->brother = atoken;
      atoken->father = ptoken->father->father;
      atoken->level  = wlevel;
      ptoken->son = NULL;
    }else{
      ptoken->brother = atoken;
      atoken->father = ptoken->father;
      atoken->level  = wlevel;
      ptoken->son = NULL;
    }			
    ptoken = atoken;
  }		

  // Then we reverse the sense of the each element inside each order of the directive modinspace or spaceintraj.
  //printf("\nAVANT\n"); lirtokgr(rootToken); // for see/debug 
  reverseRootSon();			    // We reverse firstly just the list of the root's son (it's a particular case if it is more then one element)
  reverseTokenGraph(rootToken->son);    // And the reverse beginning from the root's son 
  //printf("\nAPRES\n"); lirtokgr(rootToken); // for see/debug  		

  // Finally we go trough the new reversed graph to set the vector orderTokens. We begin from the root's son
  orderTokens.push_back("");  // We add an element in the vector because otherwise we have a segmentation fault due 
  // to the add of the exceptional element "forder" on the graph in the method reverseRootSon().

  vector<string>::iterator token=orderTokens.begin();
  setReversedVector(rootToken->son, &token);

  // Elimination of the last element added before only to avoid the generation of a segmentation fault dued to the exceptional element forder
  orderTokens.pop_back();

  // Just for test
  // for(vector<string>::iterator token=orderTokens.begin(); token < orderTokens.end(); token++) cout << endl << " token:" << *token; cout << endl;
}

/*
 * Allow to reverese the list or the root's son.
 * This list is very often of one simple element but sometime is more then one.
 * Here an example of more then one root's son:
 * order modinspace space
 *	order YA1 YA2
 *	  A B 
 *	forder
 *	order YA1 YA2
 *	  C D
 *	forder
 * forder.
 * The two order are directly connected to the root so they are the root's sons.
 */
void Order::reverseRootSon(){	
  struct st_token *p1, *p2, *pe, *ftoken;

  ftoken = (struct st_token*) malloc(sizeof(struct st_token));
  ENFORCE(ftoken)("order reverseRootSon() ftoken: error on malloc \n");

  sprintf(ftoken->token, "forder");// End the sequence of the root's son list adding a forder
  ftoken->brother = NULL;
  ftoken->father = NULL;
  ftoken->son = NULL;
  ftoken->level = 1;  // or 0 

  p1=rootToken->son;
  p2=p1->brother;

  while(p2!=NULL){		
    pe = p2;
    p2 = p2->brother;
    pe->brother = p1;
    p1 = pe;
  }
  rootToken->son->brother = ftoken;
  rootToken->son=p1;
}

/*
 * Reverse all the token graph, it's a recursive method.
 * @param aToken a node of the graph.
 */
void Order::reverseTokenGraph(struct st_token *atoken){       
  //printf("IN INVTOKGR  [%s] \n", atoken->token); 
  if(atoken == NULL) return;
  reverseTokenList(atoken->son);
  reverseTokenGraph(atoken->son);
  reverseTokenGraph(atoken->brother);
}

/*
 * Reverse the list of the sons of one father in the graph of tokens. 
 * To understand well it's important to remember the exect syntax of the order directive and to paint two or three examples of the graph.
 * The syntax of the order is:
 * ORD 		: 'order', Axe, List_AXE, Mod, List_inst , 'forder'
 * List_AXE	: vide | Axe, List_AXE
 * Axe		: YA1 | YA2 | YA3 | YB1 | YB2 | YB3
 * List_inst 	: vide | Mod, List_inst | ORD, list_inst
 * Mod		: doit exister parmi les inner de l'utilisateur 
 * @param aToken a node of the graph.
 */
void Order::reverseTokenList(struct st_token *atoken){	
  struct st_token *q0, *q1;	    // Pointer on the begin of the list_inst of the chain. 
  struct st_token *p1, *p2, *pe;// Pointer that allow the reverse of the list elements and that progress each time 

  //printf ("A reverseTokenList [%s] \n", atoken->token);	//see/debug 
  if (atoken == NULL) return;
  p1=atoken; p2=atoken->brother;

  while(1){
    if( (theModulTable->find(p1->token) || ( strcmp(p1->token,"order")==0 || strcmp(p1->token,"ORDER")==0 )) && 
	(theModulTable->find(p2->token) || ( strcmp(p2->token,"order")==0 || strcmp(p2->token,"ORDER")==0 )) ){	 
      // To be clearer: if p1 and P2 are modules or "order"
      // then we reverse the pointers and we progress p1 and p2 in the list 
      pe = p2;
      p2 = p2->brother;
      pe->brother = p1;
      p1 = pe;
    }else if(theModulTable->find(p2->token) || ( strcmp(p2->token,"order")==0 || strcmp(p2->token,"ORDER")==0 )){	 
      // To be clearer: if p2 is a module or an order then (and we were not in the precedent if)
      // we are at the begin of one list list_inst (before we should have some YAi and YBi that are not concerned from the inversion. 
      // In this case we just progress in the list but we have to remember the initial position in order to connect later. 
      q1 = p2;
      q0 = p1;
      p2 = p2->brother;
      p1 = p1->brother;
    }else if(strcmp(p2->token,"forder")==0 || strcmp(p2->token,"FORDER")==0){	 
      // We are at the end of the list...we connect the first to the last and end of the loop
      q1->brother = p2;		
      q0->brother = p1;
      break;
    }else if( (p1->token[0]=='Y' || p1->token[0]=='y') && (p2->token[0]=='Y' || p2->token[0]=='y')){	 
      // We are probably on 2 axes; we progress and that's all 
      p1 = p2;
      p2 = p2->brother;
    }else
      // Error, but it should never happen because of the grammar
      ENFORCE(false)("fail: sorry, [2]unpredictable mistake [")(p2->token)("] [")(p1->token)("]  \n")
	("possible cause: reference to Axe not precedeed by the 'order' token\n");
  }
}

/*
 * We go trough the new reversed graph to set the vector orderTokens.
 * @param aToken a node of the graph.
 * @param token the element to be changed of the orderTokens vector.
 */
void Order::setReversedVector(struct st_token *atoken, vector<string>::iterator *token){       
  //printf(" IN setReversedVector [%s] \n", atoken->token); //see/debug 
  //string str("a");
  if(atoken == NULL) return;
  **token=atoken->token;
  //*token=str;
  //cout << " " << **token;
  //sprintf(atoken->token, "a");
  //atoken->token='a';
  if(atoken->son != NULL){++*token; setReversedVector(atoken->son, token);}
  if(atoken->brother != NULL /*&& token<orderTokens.end()*/){++*token; setReversedVector(atoken->brother, token);}
}

/*
 * Recursive method for the visualzation of the graph created in order to reverse the vector that contains the tokens.
 * This method is needed just for debug.
 * @param atoken the actual token we are visualizing.
 */
void Order::lirtokgr (struct st_token *atoken){       
  // printf(" IN LIRTOKGR [%s] \n", atoken->token); // see/debug 
  if (atoken == NULL) return;
  printf ("\n {%d :%s} ", atoken->level, atoken->token);
  if (atoken->son  != NULL) lirtokgr(atoken->son);
  if (atoken->brother != NULL) lirtokgr(atoken->brother);
}






// -------------------------------------------------------------------------------------------------------------------------------- 
// ----------------------------------------------       AJOUT CB (TEST)     -------------------------------------------------------
// --------------------------------------------------------------------------------------------------------------------------------

void Order::checkOrder(Table<Connection> * theConnectionTable, Table<Modul> * theModulTable, 
    Table<Space> * theSpaceTable , Table<Operator> * theOperatorTable, Table<Order> * theOrderTable){
  //theModTable = aModulTable;
  //theConnectionTable = aConnectionTable;
  //theSpaceTable = aSpaceTable;
  
  setLoop(theConnectionTable, theModulTable, theSpaceTable, theOperatorTable, theOrderTable);
int i;
}
/*

// Function for getting an iterator on the module given as parameter (tool function)
// ---------------------------------------------------------------------------------

Table<Modul>::iterator getModule(string nom) {

        for(Table<Modul>::iterator tok_tab = theModulTable.begin(); tok_tab < theModulTable.end(); tok_tab++) {
        if(tok_tab->getName() == nom) return tok_tab;
        }
};

// Function for getting the grid size of the space "space" for the dimension "dim" (tool function)
// -----------------------------------------------------------------------------------------------

int getDimSize(string space, int dim) {
           for (Table<Operator>::iterator tok_space = theSpaceTable.begin(); tok_space != theSpaceTable.end(); ++tok_space) {
                if(tok_space -> getName() == space) return tok_space -> getYA(dim + 1);
                }
                }


// Function for getting informations about the loops and recording them in the dedicated data structures embedded inside the YAOObject "Module" 
// --------------------------------------------------------------------------------------------------------------------------------------------
*/
void Order::setLoop(Table<Connection> * theConnectionTable, Table<Modul> * theModulTable, 
    Table<Space> * theSpaceTable, Table<Operator> * theOperatorTable, Table<Order> * theOrderTable){
}
/*	string outLoopChecking = "LoopCheck_" + getFilename();
	ofstream loopFile(outLoopChecking.c_str(), ios::out);

        loopFile << "setLoop()" << endl;

        int orderCmpt = 0; 											// Number of order tokens
	int nbTraj = 0;												// Number of trajectories encountered in the order statements
	int nbSpace;												// Number of spaces encountered in the order statements

for(Table<Order>::iterator tok_tab = theOrderTable->begin(); tok_tab < theOrderTable->end(); tok_tab++) {
	 if(tok_tab -> getOrderPhase() == 2) {
			nbTraj++; 
			nbSpace = 0; 	
			for(vector<string>::iterator tok_str = tok_tab -> orderTokens.begin(); tok_str < tok_tab -> orderTokens.end(); tok_str++) { nbSpace++; } 
			}
			} 
}			
string indexSpaceTraj[nbTraj][nbSpace];					// Table for storing spaces inside trajectories (used for ordonnancing the spaces as they are declared in the spaceintraj statements)

for(int i = 0; i < nbTraj; i++) { for(int j = 0; j < nbSpace; j++) { indexSpaceTraj[i][j] = ""; } }  // Initialization of indexSpaceTraj table with zero length string 

nbTraj =  0;

for(Table<Order>::iterator tok_tab = theOrderTable.begin(); tok_tab < theOrderTable.end(); tok_tab++) {  // Reading of the order table for populating the indexSpaceTraj table
		tok_tab->backOrder();									 // For sorting the tokens as they appear in the description file
		if(tok_tab -> getOrderPhase() == 2) {							 // If the table entry is a spaceintraj statement
			nbTraj++;
			nbSpace = 0;
                	for(vector<string>::iterator tok_str = tok_tab -> orderTokens.begin(); tok_str < tok_tab -> orderTokens.end(); tok_str++) {
				   indexSpaceTraj[nbTraj -1][nbSpace++] = *tok_str;
                        }
                        }
			} 

for(Table<Order>::iterator tok_tab = theOrderTable.begin(); tok_tab < theOrderTable.end(); tok_tab++) {				     			// For all entries in the tale Order
	if(tok_tab -> getOrderPhase() == 1) {														// If the entry is a modinspace
		for(vector<string>::iterator tok_str = tok_tab -> orderTokens.begin(); tok_str < tok_tab -> orderTokens.end(); tok_str++) { 		// Read the vector of tokens
			if (*tok_str == "order") orderCmpt++;										     
			}	
			}
			} 

        int *nbTokens           = new int[orderCmpt];                                   // Number of "Y.." tokens (indexed by [no_loop])
        int loopId[3][3]        = { { 0, 0, 0 }, { 0, 0, 0 }, { 0, 0, 0 } };            // Loop Id (indexed by [loopOrder][loopId])  
        int directionPath[3][3] = { { 0, 0, 0 }, { 0, 0, 0 }, { 0, 0, 0 } };            // Path Direction (indexed by [loopOrder][tokenCmpt]) 
        int axisNumber[3][3]    = { { 0, 0, 0 }, { 0, 0, 0 }, { 0, 0, 0 } };            // Dimension number (indexed by [loopOrder][tokenCmpt])
        int loopOrder = -1;                                                             // Recorder of the order's appearing number (managment of overlapped loops)
        int loopCmpt  = 0;                                                              // Recorder of the Number of loops.
        int tokenCmpt = -1;                                                             // Number of "Y.." tokens in an order loop (max. 3, managments of not overlapped loops)
        int modulCmpt = 0;                                                              // Module's sequence Id  


for(int i = 0; i < nbTraj; i++) {                                                                                                            // For all trajectories "i" indexed in indexSpaceTraj 
     for(int j = 0; j < nbSpace; j++) {                                                                                                      // For all spaces "j" stored in the current trajectory "i"
        for(Table<Order>::iterator tok_tab = theOrderTable.begin(); tok_tab < theOrderTable.end(); tok_tab++) {                              // For all entries in the table Order
            if(tok_tab -> getOrderPhase() == 1 && tok_tab -> getName() == indexSpaceTraj[i][j]) { 					     // If the entry is a modinspace and the space match indexSpaceTraj
                for(vector<string>::iterator tok_str = tok_tab -> orderTokens.begin(); tok_str < tok_tab -> orderTokens.end(); tok_str++) {  // Read the vector of tokens

if (*tok_str == "order")        { loopOrder++; loopCmpt++; tokenCmpt = -1; }
else if (*tok_str == "forder")  { loopOrder--;                                                                                                                                                               }
else if (*tok_str == "YA1")     { tokenCmpt++; nbTokens[loopOrder] = tokenCmpt + 1; loopId[loopOrder][tokenCmpt] = loopCmpt; directionPath[loopOrder][tokenCmpt] = -1; axisNumber[loopOrder][tokenCmpt] = 1; }
else if (*tok_str == "YA2")     { tokenCmpt++; nbTokens[loopOrder] = tokenCmpt + 1; loopId[loopOrder][tokenCmpt] = loopCmpt; directionPath[loopOrder][tokenCmpt] = -1; axisNumber[loopOrder][tokenCmpt] = 2; }
else if (*tok_str == "YA3")     { tokenCmpt++; nbTokens[loopOrder] = tokenCmpt + 1; loopId[loopOrder][tokenCmpt] = loopCmpt; directionPath[loopOrder][tokenCmpt] = -1; axisNumber[loopOrder][tokenCmpt] = 3; }
else if (*tok_str == "YB1")     { tokenCmpt++; nbTokens[loopOrder] = tokenCmpt + 1; loopId[loopOrder][tokenCmpt] = loopCmpt; directionPath[loopOrder][tokenCmpt] =  1; axisNumber[loopOrder][tokenCmpt] = 1; }
else if (*tok_str == "YB2")     { tokenCmpt++; nbTokens[loopOrder] = tokenCmpt + 1; loopId[loopOrder][tokenCmpt] = loopCmpt; directionPath[loopOrder][tokenCmpt] =  1; axisNumber[loopOrder][tokenCmpt] = 2; }
else if (*tok_str == "YB3")     { tokenCmpt++; nbTokens[loopOrder] = tokenCmpt + 1; loopId[loopOrder][tokenCmpt] = loopCmpt; directionPath[loopOrder][tokenCmpt] =  1; axisNumber[loopOrder][tokenCmpt] = 3; }
else 				{

        getModule(*tok_str) -> initLoopInfos();                                         // Initialisation of all loops informations
        modulCmpt++;									// Increment of the modules counter 
        int tabIndex = -1;                                                              // Position Index in the module's tables 

for(int i = 0; i <= loopOrder; i++) {
        for(int j = 0; j < nbTokens[i]; j++) {
                tabIndex++;
                getModule(*tok_str) -> setSeqId(modulCmpt);
                getModule(*tok_str) -> setAxisNumber(tabIndex, axisNumber[i][j]);
                getModule(*tok_str) -> setAxisVal(tabIndex, directionPath[i][j]);
                getModule(*tok_str) -> setLoopId(tabIndex, loopId[i][j]);
                }
                }



                                }
                                }
                                }
                                }
				}
				}
				} 


// Function for testing all the connections of the description file (main entry of the procedure)
// ---------------------------------------------------------------------------------------------- 

void checkConnections() {

	string outLoopChecking = "LoopCheck_" + getFilename();
        fstream loopFile(outLoopChecking.c_str(), fstream::in | fstream::out | fstream::app);

        loopFile << endl << "checkConnections()";

        for(Table<Connection>::iterator tok_tab = theConnectionTable.begin(); tok_tab < theConnectionTable.end(); tok_tab++) {

	if(tok_tab -> getT() == -1) { 
	loopFile << endl << "Dropped T-1  : " << tok_tab -> getOutModule() << " (" << tok_tab -> getI() << "," << tok_tab -> getJ() << "," << tok_tab -> getK() << "," << tok_tab -> getT() <<  
	        ") ==> " << tok_tab -> getInModule();
	} 

	if(tok_tab -> getT() > -1) { 										// Drop connection if the dependencies are not defined at current time

	Table<Modul>::iterator itSource = getModule(tok_tab -> getOutModule());
	Table<Modul>::iterator itTarget = getModule(tok_tab -> getInModule());

	if(!itSource -> isCounterOrder() || !itTarget -> isCounterOrder()) { 
	loopFile << endl << "Dropped Order : " << tok_tab -> getOutModule() << " (" << tok_tab -> getI() << "," << tok_tab -> getJ() << "," << tok_tab -> getK() << ") ==> " << tok_tab -> getInModule();
	}

	if(itSource -> isCounterOrder() && itTarget -> isCounterOrder())  { 					// Drop connection if modules are not integrated in an order statement

	if(itSource -> getTrajectory() != itTarget -> getTrajectory())    {
	loopFile << endl << "Dropped Trajectory : " << tok_tab -> getOutModule() << " (" << tok_tab -> getI() << "," << tok_tab -> getJ() << "," << tok_tab -> getK() << ") ==> " << tok_tab -> getInModule();
	}

	if(itSource -> getTrajectory() == itTarget -> getTrajectory())    {					// Drop connection if modules are not in the same trajectory 
	
	int valCijk[3];

        valCijk[0] = tok_tab -> getI();
        valCijk[1] = tok_tab -> getJ();
        valCijk[2] = tok_tab -> getK();
	
        loopFile << endl << endl << "Connexion : " << tok_tab -> getOutModule() << " (" << valCijk[0] << "," << valCijk[1] << "," << valCijk[2] <<") ==> " << tok_tab -> getInModule();
        loopFile << "   (" <<  itSource -> getAxisNumber(0) << ", " <<  itSource -> getAxisNumber(1) << ", " <<  itSource -> getAxisNumber(2) << ")" << endl;
        loopFile << "-----------------------------------------------" << endl;

	// Compatibility test between the order statement and the dimensions of the source module

      	if(itSource -> isSetAxis(1) == true  && getDimSize(itSource -> getSpaceOrOperator(), 0) == 0) loopFile << endl << "Warning : computation of I dimension not necessary" << endl;
	if(itSource -> isSetAxis(1) == false && getDimSize(itSource -> getSpaceOrOperator(), 0) > 0)  loopFile << endl << "Error   : dimension I must be computed" << endl;
      	if(itSource -> isSetAxis(2) == true  && getDimSize(itSource -> getSpaceOrOperator(), 1) == 0) loopFile << endl << "Warning : computation of J dimension not necessary" << endl;
      	if(itSource -> isSetAxis(2) == false && getDimSize(itSource -> getSpaceOrOperator(), 1) > 0 && itSource -> isInterYA(1) == false) loopFile << endl << "Error   : dimension J must be computed" << endl;
      	if(itSource -> isSetAxis(3) == true  && getDimSize(itSource -> getSpaceOrOperator(), 2) == 0) loopFile << endl << "Warning : computation of K dimension not necessary" << endl;
     	if(itSource -> isSetAxis(3) == false && getDimSize(itSource -> getSpaceOrOperator(), 2) > 0 && itSource -> isInterYA(2) == false) loopFile << endl << "Error   : dimension K must be computed" << endl;
	// Data structures for testing absolute references
	
        bool absCijk[3];

        absCijk[0] = false;      //  Flag indicating an absolute i value
        absCijk[1] = false;      //  Flag indicating an absolute j value 
        absCijk[2] = false;      //  Flag indicating an absolute k value
        if(tok_tab -> getReli() == -1) { absCijk[0] = true; }  
        if(tok_tab -> getRelj() == -1) { absCijk[1] = true; } 
        if(tok_tab -> getRelk() == -1) { absCijk[2] = true; }  

        int lastAxis = 2;                                                                                  // Without more informations, 3 axis are taken into account 
        int Res = 0;                                                                                       // Res is initialised at 0, means there is no information about order validity 

        for(int i = 0; i < 3; i++) {

		if(Res != 0) break; 						// If checkConnection() validate are invalidate the order statement for current connection, no further tests are needed 

                        if(i < 3 && itSource -> getAxisNumber(i + 1) == 0) lastAxis = i;
                        if(itSource -> getAxisNumber(i) > 0)
                        Res = checkConnection(i, itSource, itTarget, valCijk, absCijk, lastAxis);

                        }
        }
        }
	}
	}
	loopFile.close();
        }



// Function, called by checkConnections(), used for testing the dimension "numDim"
// -------------------------------------------------------------------------------

bool checkConnection(int numDim, Table<Modul>::iterator itSource, Table<Modul>::iterator itTarget, int valCijk[3], bool absCijk[3], int lastAxis)  {

	string outLoopChecking = "LoopCheck_" + getFilename();
        fstream loopFile(outLoopChecking.c_str(), fstream::in | fstream::out | fstream::app);

        loopFile << endl << "checkConnection(" << numDim << ")";

// If modules are not in the same loop for the current dimension, then the odering of modules is checked

        if(itSource -> getLoopId(numDim) != itTarget -> getLoopId(numDim)) {  loopFile << " Squence : 1 => 2 "; return checkModOrder(itSource, itTarget);   }

 
// Else, check if the value for current dimension is an absolute one and, if so, test the validity of the order


string  msgWarning = " (Warning: Absolute reference can not be satisfied)";

if(absCijk[itSource -> getAxisNumber(numDim) - 1] == true) {

if(itSource -> getAxisVal(numDim) < 0 && valCijk[itSource -> getAxisNumber(numDim) - 1] < 2 && numDim != lastAxis) { loopFile << " Squence : 1 => 3 => 4 => 5 => O.K. (dim)" << endl; return 0; }
if(itSource -> getAxisVal(numDim) < 0 && valCijk[itSource -> getAxisNumber(numDim) - 1] < 2 && numDim == lastAxis) { loopFile << " Squence : 1 => 3 => 4 => 5 => 2";  return  checkModOrder(itSource, itTarget);  }
if(itSource -> getAxisVal(numDim) < 0 && valCijk[itSource -> getAxisNumber(numDim) - 1] > 1 && numDim != lastAxis) { loopFile << " Squence : 1 => 3 => 4 => 5 => O.K.(dim)" << msgWarning << endl; return 0; }
if(itSource -> getAxisVal(numDim) < 0 && valCijk[itSource -> getAxisNumber(numDim) - 1] > 1 && numDim == lastAxis) { loopFile << " Squence : 1 => 3 => 4 => 5 => 2" << msgWarning; return  checkModOrder(itSource, itTarget);  }

if(itSource -> getAxisVal(numDim) > 0 && valCijk[itSource -> getAxisNumber(numDim) - 1] > getDimSize(itSource -> getSpaceOrOperator(), numDim) - 2 && numDim != lastAxis) { 
														loopFile << " Squence : 1 => 3 => 4 => 5 => O.K. (dim)" << endl; 
														return 0;  
														}
if(itSource -> getAxisVal(numDim) > 0 && valCijk[itSource -> getAxisNumber(numDim) - 1] > getDimSize(itSource -> getSpaceOrOperator(), numDim) - 2 && numDim == lastAxis) { 
														loopFile << " Squence : 1 => 3 => 4 => 5 => 2"; 
														return  checkModOrder(itSource, itTarget); 
														}
if(itSource -> getAxisVal(numDim) > 0 && valCijk[itSource -> getAxisNumber(numDim) - 1] < getDimSize(itSource -> getSpaceOrOperator(), numDim) - 1 && numDim != lastAxis) {  
														loopFile << " Squence : 1 => 3 => 4 => 5 => O.K. (dim)" << msgWarning << endl; 
														return 0;
														}
if(itSource -> getAxisVal(numDim) > 0 && valCijk[itSource -> getAxisNumber(numDim) - 1] < getDimSize(itSource -> getSpaceOrOperator(), numDim) - 1 && numDim == lastAxis) {
														loopFile << " Squence : 1 => 3 => 4 => 5 => 2" << msgWarning;
														return  checkModOrder(itSource, itTarget);
														}

}


// Else, if modules are in the same node for the current dimension

if(valCijk[itSource -> getAxisNumber(numDim) - 1] == 0) {


                // If the current dimension is not the last one defined in the connection, the order is validated for the current dimension 

                if(numDim != lastAxis) {  loopFile << " Squence : 1 => 3 => 6 => 5 => O.K. (dim)" << endl; return 0; }

                // Else the odering of modules is checked 

                else   { loopFile << " Squence : 1 => 3 => 6 => 5 => 2"; return checkModOrder(itSource, itTarget);  }

                }


// Else, test the validity between order statements and connections 


if(itSource -> getAxisVal(numDim) < 0 && valCijk[itSource -> getAxisNumber(numDim) - 1] < 0) { loopFile << " Squences : 1 => 3 => 6 => 7 => O.K." << endl; return 1;  }
if(itSource -> getAxisVal(numDim) < 0 && valCijk[itSource -> getAxisNumber(numDim) - 1] > 0) { loopFile << " Squences : 1 => 3 => 6 => 7 => K.O." << endl; return -1; }
if(itSource -> getAxisVal(numDim) > 0 && valCijk[itSource -> getAxisNumber(numDim) - 1] < 0) { loopFile << " Squences : 1 => 3 => 6 => 7 => K.O." << endl; return -1; }
if(itSource -> getAxisVal(numDim) > 0 && valCijk[itSource -> getAxisNumber(numDim) - 1] > 0) { loopFile << " Squences : 1 => 3 => 6 => 7 => O.K." << endl; return 1;  }

}

// Function, called by checkConnection(), used for testing the ordering of the modules given as parameters
// -------------------------------------------------------------------------------------------------------

bool checkModOrder(Table<Modul>::iterator itSource, Table<Modul>::iterator itTarget) {
	
	string outLoopChecking = "LoopCheck_" + getFilename();
        fstream loopFile(outLoopChecking.c_str(), fstream::in | fstream::out | fstream::app);

        if(itSource -> getSeqId() <= itTarget -> getSeqId()) { loopFile << " => O.K." << endl;  return 1; }
        loopFile << " => K.O." << endl; return -1;
        }


*/


// ------------------------------------ DEBUT AJOUT CB - GENERATION AUTOMATIQUE DE L'ORDER -------------------------------------------


void Order::automaticOrderGenerator() {


	/*
         * Get informations on the trajectories then map all the spaces with the trajectory in which they are included. 
         * As this information is not provided by the space (the space object have no information about the trajectories),
         * the mapping is done by extracting spaces and trajectories informations from the moduls. 
         */

        // Map that will contain the space name as a key and the trajectory name as a matched value
        // ----------------------------------------------------------------------------------------

        map<string, string> trajMap;

        // Vector that will contain the trajectories name
        // ----------------------------------------------

        vector<string> trajVector;

        // For each modul 
        // --------------

        for(Table<Modul>::iterator tok_tab = myModulTable -> begin(); tok_tab < myModulTable -> end(); tok_tab++) {

                // If the module is not a noward one
                // ---------------------------------

                if(!tok_tab -> isNoward()) {

                        // By default the trajectory is considerated as a new one 
                        // ------------------------------------------------------

                        bool isSetTraj = false;

                        // For all the trajectories already integrated in the vector 
                        // ---------------------------------------------------------

                        for(int noTraj = 0; noTraj < trajVector.size(); noTraj++)

                                // If the trajectory match the current one, then it should be considerated as already integrated 
                                // ---------------------------------------------------------------------------------------------

                                if(!isSetTraj && trajVector[noTraj] == tok_tab -> getTrajectory()) isSetTraj = true;

                        // If the trajectory is really a new one, then add it in the trajectories vector
                        // -----------------------------------------------------------------------------

                        if(!isSetTraj) trajVector.push_back(tok_tab -> getTrajectory());

                        // Update the trajectory map with the current couple space / trajectory
                        // --------------------------------------------------------------------

                        trajMap[tok_tab -> getSpaceOrOperator()] = tok_tab -> getTrajectory();

                }
        }

	
        /*
         * Set up a graph that will contain one vertex by space and one edge by pair of moduls connected from different spaces. 
         * As this graph can not contain any clycle, a topological sort is performed to know in which order the spaces should 
         * be intergated in the spaceintraj statement. 
         */

        // Vector that will contain the space names
        // ----------------------------------------

        vector<string> spacesList;

        // For all spaces of the description file, add the space name in the space list 
        // ----------------------------------------------------------------------------

        for (Table<Operator>::iterator tok_space = mySpaceTable -> begin(); tok_space != mySpaceTable -> end(); ++tok_space) spacesList.push_back(tok_space -> getName());

        // Create a graph that will be used to sort the spaces
        // ---------------------------------------------------

        myGraphType spaceGraph;

        // Add one vertex by space
        // -----------------------

        for(int i = 0; i < spacesList.size(); i++) add_vertex(spaceGraph);

        // Register the space name inside each vertex
        // ------------------------------------------

        for(tie(vertexIterOne, vertexIterTwo) = vertices(spaceGraph); vertexIterOne != vertexIterTwo; ++vertexIterOne) put(get(boost::vertex_name_t(), spaceGraph), *vertexIterOne, spacesList[*vertexIterOne]);

        // For all the connections of the description file
        // -----------------------------------------------

        for(Table<Connection>::iterator tok_tab = myConnectionTable -> begin(); tok_tab < myConnectionTable -> end(); tok_tab++) {

                // Select just the dependencies defined on the current time
                // ---------------------------------------------------------

                if(tok_tab -> getT() > -1) {

                        if(!getModule(tok_tab -> getOutModule()) -> isNoward() && !getModule(tok_tab -> getInModule()) -> isNoward()) {

                        // If the source modul is not in the same space than the target modul
                        // ------------------------------------------------------------------

                        if(getModule(tok_tab -> getOutModule()) -> getSpaceOrOperator() != getModule(tok_tab -> getInModule()) -> getSpaceOrOperator()) {

                                // Add an edge between the space in which the source modul is included and the space in which the target modul is included
                                // -----------------------------------------------------------------------------------------------------------------------

                                int spacePairOne = getModulIndex(getModule(tok_tab -> getOutModule()) -> getSpaceOrOperator(), spacesList, spacesList.size());

                                int spacePairTwo = getModulIndex(getModule(tok_tab -> getInModule()) -> getSpaceOrOperator(), spacesList, spacesList.size());

                                add_edge(spacePairOne, spacePairTwo, spaceGraph);

                                }
                        }
                }
        }

        // Data structure that will be used by boost to register the topological sort informations
        // ---------------------------------------------------------------------------------------

        std::deque<int> topoOrder;

        // Perform the topological sort
        // ----------------------------

         boost::topological_sort(spaceGraph, std::front_inserter(topoOrder), boost::vertex_index_map(boost::identity_property_map()));


        /*
         * Accordignly to the space ordonnancing that just have been performed, set up an order for the trajectories.
         * This step could be avoided as it appears that the trajectories can be integrated in any order (for now this step is integrated). 
         * The sorting criteria is the order of the latest space integrated in the trajectory (space order performed over all the trajectories). 
         * The trajectory which the latest space has the lower order Id (in the main space ordonnancing) will be integrated in first. 
         * The trajectory which the latest space has the higher order Id (in the main space ordonnancing) will be integrated in last. 
         */

       typedef pair<int, string> trajOrderType;

       vector<trajOrderType> trajOrder;

        // For all the trajectories
        // ------------------------- 

        for(int noTraj = 0; noTraj < trajVector.size(); noTraj++) {

                int orderId = -1;

                // For all the spaces
                // ------------------

                for(int noSpace = 0; noSpace < topoOrder.size(); noSpace++) {

                        // In the order that have been defined to the spaces by the topological sort 
                        // -------------------------------------------------------------------------

                        int noTopo = topoOrder[noSpace];

                        // If the current space is a part of the current trajectory 
                        // --------------------------------------------------------

                        if(trajMap[spacesList[noTopo]] == trajVector[noTraj]) {

                                // Update the orderId value with the current noSpace value
                                // -------------------------------------------------------

                                orderId = noSpace;
                        }
                }

                // Set to the trajectory is orderId (i.e : order of the latest space that is included in this trajectory) 
                // ------------------------------------------------------------------------------------------------------

                trajOrder.push_back(trajOrderType(orderId,  trajVector[noTraj]));
        }

	// Sort the trajectories accordingly to their higher space order Id 
	// ----------------------------------------------------------------

	sort(trajOrder.begin(), trajOrder.end()); 


	/*
	 * Launch the modinspace generating space by space in the order that have been defined in the spaceintraj generating step just below
	 */

	//  For all the sorted trajectories
        // --------------------------------

        for(int noTraj = 0; noTraj < trajOrder.size(); noTraj++) {

                // For all the space numbers 
                // -------------------------

                for(int noSpace = 0; noSpace < topoOrder.size(); noSpace++) {

                        // In the order that have been defined to the spaces by the topological sort 
                        // -------------------------------------------------------------------------

                        int noTopo = topoOrder[noSpace];

                        // If the trajectory affected to the sorted space is the current trajectory
                        // ------------------------------------------------------------------------

                        if((trajMap.find(spacesList[noTopo]) -> second).compare(trajOrder[noTraj].second) == 0) {

                                // Launch the modinspace generation for the current space  
                                // ------------------------------------------------------

				string spaceName = trajMap.find(spacesList[noTopo]) -> first;

				// Show the modinspace line
				// ------------------------

				cout << endl << endl << "order modinspace " << spaceName << endl;

   				// Dedicated light structures that will be used to create the main graph then populate its properties 
   				// ---------------------------------------------------------------------------------------------------

   				vector<edge> edgeArray;
   				vector<string> modulList;
   				vector<int> valCi, valCj, valCk;

   				// Array of boolean values that will be used to know the dimensions on which the moduls are defined
   				// ------------------------------------------------------------------------------------------------

  				 bool *isDefinedOn = new bool[3];

   				isDefinedOn[0] = false;
   				isDefinedOn[1] = false;
   				isDefinedOn[2] = false;

   				// Scan the table of moduls to get relevable informations for setting up the graph and get informations on dimensions
   				// ------------------------------------------------------------------------------------------------------------------

   				for(Table<Modul>::iterator tok_tab = myModulTable -> begin(); tok_tab < myModulTable -> end(); tok_tab++) {

					// If the modul is in the current space
       					// ------------------------------------

        				if(tok_tab -> getSpaceOrOperator() == spaceName) {

  						// Select just the moduls not involved in noward
  						// ---------------------------------------------

  						if(!tok_tab ->isNoward()) {

 							// Get informations on modul names then populate the modulList vector that will be used by Boost to register the vertex_name property
 							// ----------------------------------------------------------------------------------------------------------------------------------

							modulList.push_back(tok_tab -> getName());

  							// Get informations on dimensions where are defined the moduls then populate the isDefinedOn array
  							// -----------------------------------------------------------------------------------------------

               						if(tok_tab -> getYA(1) > 0) isDefinedOn[0] = true;
               						if(tok_tab -> getYA(2) > 0) isDefinedOn[1] = true;
               						if(tok_tab -> getYA(3) > 0) isDefinedOn[2] = true;
         					} 
   					} 
				} 
			
   				// Avoid to work on an empty set
   				// -----------------------------

   				if(modulList.size() > 0) {	


					// Scan the table of the connections to get all relevable informations for generating the order statement
					// ------------------------------------------------------------------------------------------------------

        				for(Table<Connection>::iterator tok_tab = myConnectionTable -> begin(); tok_tab < myConnectionTable -> end(); tok_tab++) {

						// Select just the dependencies defined on the current time
						// ---------------------------------------------------------

                				if(tok_tab -> getT() > -1) {

                        				Table<Modul>::iterator itSource = getModule(tok_tab -> getOutModule());
                        				Table<Modul>::iterator itTarget = getModule(tok_tab -> getInModule());


							        // If both source and target modules are in the current space 
        							// ----------------------------------------------------------

                        					if(itSource -> getSpaceOrOperator() == spaceName && itTarget -> getSpaceOrOperator() == spaceName) {


									// Select just the moduls not involved in noward (match the condition used to create the modulList vector)
									// -------------------------------------------------------------------------------------------------------

                        						if(!itSource -> isNoward() && !itTarget -> isNoward() ) {

                                						string sourceName = itSource -> getName();
                                						string targetName = itTarget -> getName();

										// Setting all pair of integer values that will  be used by Boost to manage the edges
										// ----------------------------------------------------------------------------------

                                						int modPairOne = getModulIndex(itSource -> getName(), modulList, modulList.size());
                                						int modPairTwo = getModulIndex(itTarget -> getName(), modulList, modulList.size());
	
										edgeArray.push_back(edge(modPairOne, modPairTwo));

										// Getting informations about spatial dependency between moduls for the dimension i then populate the dedicated vector
										// -------------------------------------------------------------------------------------------------------------------

										valCi.push_back(tok_tab -> getI());

										// Getting informations about spatial dependency between moduls for the dimension j then populate the dedicated vector
										// -------------------------------------------------------------------------------------------------------------------

										valCj.push_back(tok_tab -> getJ());

										// Getting informations about spatial dependency between moduls for the dimension k then populate the dedicated vector
										// -------------------------------------------------------------------------------------------------------------------

										valCk.push_back(tok_tab -> getK());
	
									}
                        					}
                					}
        					}   

					// Get the number of dimensions on which tho moduls are defined
					// ------------------------------------------------------------

					switch(isDefinedOn[0] + isDefinedOn[1] + isDefinedOn[2]) {

					// If the moduls are defined on on dimension 
					// -----------------------------------------
	
					case 1 : 

					// Modify the spatial dependencies vector to offset i dimension to the k dimension then set the dependencies to zero for i and j dimensions
					// ----------------------------------------------------------------------------------------------------------------------------------------

					to3D(1, valCi, valCj, valCk, edgeArray.size());


					resetGraph(myGraph);

					// Create the main graph with the informations collected (and modified for spatial dependencies) by scanning YAO tables
					// --------------------------------------------------------------------------------------------------------------------

 					generateGraph(myGraph, valCi, valCj, valCk, modulList, edgeArray);

 					// Launch the main function that will create the vector 3D components, the vectors of 2D components then the vectors of 1D components
					// ---------------------------------------------------------------------------------------------------------------------------------- 

	 				setOuter();

					// Read the vector of 1D components to display the order statement (the two first dimensions have dummy values, then the third one is read as the first one) 
					// ---------------------------------------------------------------------------------------------------------------------------------------------------------

					read1D();

					break;

					case 2 :

					// Modify the spatial dependencies vector to offset i and j dimensions to k j and k dimensions then set the dependencies to zero for the i dimension
					// -------------------------------------------------------------------------------------------------------------------------------------------------

					to3D(2, valCi, valCj, valCk, edgeArray.size());

					resetGraph(myGraph);

	 				// Create the main graph with the informations collected (and modified for spatial dependencies) by scanning YAO tables
        				// --------------------------------------------------------------------------------------------------------------------

					generateGraph(myGraph, valCi, valCj, valCk, modulList, edgeArray);

					// Launch the main function that will create the vector 3D components, the vectors of 2D components then the vectors of 1D components
        				// ---------------------------------------------------------------------------------------------------------------------------------- 

					setOuter();

					// Read the vector of 2D components to display the order statement (the first dimension have dummy value, then the second and third are read as the first and second)
					// ------------------------------------------------------------------------------------------------------------------------------------------------------------------

					read2D(); 

					break;

					case 3 : 

					resetGraph(myGraph);
	
					// Create the main graph with the informations collectedby scanning YAO tables
 					// --------------------------------------------------------------------------
		
					generateGraph(myGraph, valCi, valCj, valCk, modulList, edgeArray);

					// Launch the main function that will create the vector 3D components, the vectors of 2D components then the vectors of 1D components
        				// ---------------------------------------------------------------------------------------------------------------------------------- 

					setOuter();

					// Read the vector of 3D components
					// --------------------------------

					read3D();

					break;

					}
    				} 
			
				cout << endl << "forder";
			}
		}
	}


	/*
         * Perform the spaceintraj writting. 
         * For each trajectory in the trajectories order that have been performed (maybe not needed).
         * For each space of the trajectory in the space order that have been performed 
         */


        //  For all the sorted trajectories
        // --------------------------------

        for(int noTraj = 0; noTraj < trajOrder.size(); noTraj++) {

                cout << endl << endl << "order spaceintraj " << trajOrder[noTraj].second << endl << endl << "\t";

                // For all the space numbers 
                // -------------------------

                for(int noSpace = 0; noSpace < topoOrder.size(); noSpace++) {

                        // In the order that have been defined to the spaces by the topological sort 
                        // -------------------------------------------------------------------------

                        int noTopo = topoOrder[noSpace];

                        // If the trajectory affected to the sorted space is the current trajectory
                        // ------------------------------------------------------------------------

                        if((trajMap.find(spacesList[noTopo]) -> second).compare(trajOrder[noTraj].second) == 0) {

                                        // Insert the space in the spaceintraj order 
                                        // -----------------------------------------

                                        cout << trajMap.find(spacesList[noTopo]) -> first << " ";
                        }
                }


        // Close the current spaceintraj statement
        // ---------------------------------------

        cout << endl << endl <<  "forder";

        }


};



Table<Modul>::iterator Order::getModule(string nom) {

        for(Table<Modul>::iterator tok_tab = myModulTable -> begin(); tok_tab < myModulTable -> end(); tok_tab++) {
        if(tok_tab->getName() == nom) return tok_tab;
        }
        };


int Order::getModulIndex(string mod, const vector<string>& tab, int index) { for(int i = 0; i < index; i++) if(tab[i] == mod) return i; }


void Order::to3D(int nbDim, vector<int>& valCi, vector<int>& valCj, vector<int>& valCk, int nbEdges) {

                if(nbDim == 2) { for(int noEdge = 0; noEdge < nbEdges; noEdge++) {
		
				valCk[noEdge] = valCj[noEdge];
                                valCj[noEdge] = valCi[noEdge];
                                valCi[noEdge] = 0;
				}
		}

                if(nbDim == 1) { for(int noEdge = 0; noEdge < nbEdges; noEdge++) {

				valCk[noEdge] = valCi[noEdge];
                                valCj[noEdge] = 0;
                                valCi[noEdge] = 0;
				}
		}
        };



void Order::generateGraph(myGraphType& currentGraph, const vector<int>& valCi, const vector<int>& valCj, const vector<int>& valCk, const vector<string>& modulList, const vector<edge>& edgeArray) {

// INSTANCIATION OF THE GRAPH (just main components at this stage)
// ---------------------------------------------------------------

for(int i = 0; i < modulList.size(); i++) add_vertex(currentGraph);

for(int i = 0; i < edgeArray.size(); i++) add_edge(edgeArray[i].first, edgeArray[i].second, currentGraph);


// POPULATING PROPERTIES
// ---------------------

for(tie(vertexIterOne, vertexIterTwo) = vertices(currentGraph); vertexIterOne != vertexIterTwo; ++vertexIterOne) {
        put(get(boost::vertex_name_t(), currentGraph), *vertexIterOne, modulList[*vertexIterOne]);
        put(get(boost::vertex_topo_order_t(), currentGraph), *vertexIterOne, -1);
        put(get(boost::vertex_affected_comp_t(), currentGraph), *vertexIterOne, -1);
	put(get(boost::vertex_original_id_t(), currentGraph), *vertexIterOne, *vertexIterOne); 
        }

int edgesCmpt = 0;

for(tie(edgeIterOne, edgeIterTwo) = edges(currentGraph); edgeIterOne != edgeIterTwo; ++edgeIterOne) {
        put(get(boost::edge_weight_d1_t(), currentGraph), *edgeIterOne, valCi[edgesCmpt]);
        put(get(boost::edge_weight_d2_t(), currentGraph), *edgeIterOne, valCj[edgesCmpt]);
        put(get(boost::edge_weight_d3_t(), currentGraph), *edgeIterOne, valCk[edgesCmpt]);
        edgesCmpt++;
        }

};
 


int Order::getCommonSignInner(int noFixed, int noFree, myGraphType& currentGraph) {

        // Create a graph that will be fixed before to perform the sign analyzis (working graph)
        // -------------------------------------------------------------------------------------

        myGraphType fixedGraph;

        // Copy into the working graph the informations of the graph given as parameter
        // ----------------------------------------------------------------------------

        copyGraph(currentGraph, fixedGraph);

        // Fix the working graph by keeping only edges with null value for the dimension "noFixed" given as parameter
        // ----------------------------------------------------------------------------------------------------------

        fixGraph(fixedGraph, noFixed);

        // Perform a sign analyzis on the fixed graph for the dimension "noFree" given as paramater
        // ----------------------------------------------------------------------------------------

        return(getCommonSign(noFree, fixedGraph));

};



void Order::showOuter() {

        cout << endl << "PROTOTYPE : outerComp[noComp3D]" << endl;

        // For all 3D component embedded in the vector outerComp
        // -----------------------------------------------------

        for(int noComp3D = 0; noComp3D < outerComp.size(); noComp3D++) {

                // Display all relevant informations on components : dimension and sign of the three first axis
                // --------------------------------------------------------------------------------------------

                cout << endl << "outerComp[" << noComp3D << "].cfcSign[0].first : " <<  outerComp[noComp3D].cfcSign[0].first << " , second : " << outerComp[noComp3D].cfcSign[0].second;
                cout << endl << "outerComp[" << noComp3D << "].cfcSign[1].first : " <<  outerComp[noComp3D].cfcSign[1].first << " , second : " << outerComp[noComp3D].cfcSign[1].second;
                cout << endl << "outerComp[" << noComp3D << "].cfcSign[2].first : " <<  outerComp[noComp3D].cfcSign[2].first << " , second : " << outerComp[noComp3D].cfcSign[2].second;

                cout << endl;

        }
        };


void Order::showInner(int noFixed1, int noFixed2, int freeDim) {

        cout << endl << "PROTOTYPE : innerComp[noComp3D][noFixed1][noComp2D][noFixed2][noComp1D]" << endl;

        //  For all 3D component embedded in the vector outerComp
        // ------------------------------------------------------

        for(int noComp3D = 0; noComp3D < outerComp.size(); noComp3D++) {

                //  For all 2D component embedded in the vector middleComp
                // -------------------------------------------------------

                for(int noComp2D = 0; noComp2D < middleComp[noComp3D][noFixed1].size(); noComp2D++) {

                        // For all 1D component embedded in the vector innerComp
                        // -----------------------------------------------------        

                        for(int noComp1D = 0; noComp1D < innerComp[noComp3D][noFixed1][noComp2D][noFixed2].size(); noComp1D++)  {

                                // Display all relevant informations on components : dimension and sign of the remaining axis then embedded moduls
                                // ---------------------------------------------------------------------------------------------------------------


        cout    << endl << "innerComp[" << noComp3D << "][" << noFixed1 << "][" << noComp2D << "][" << noFixed2 << "][" << noComp1D << "] Dim1 : " <<
                            innerComp[noComp3D][noFixed1][noComp2D][noFixed2][noComp1D].cfcSign[0].first << " , second : " << innerComp[noComp3D][noFixed1][noComp2D][noFixed2][noComp1D].cfcSign[0].second;

        cout << endl;

                                for(int noMod = 0; noMod < innerComp[noComp3D][noFixed1][noComp2D][noFixed2][noComp1D].embeddedModuls.size(); noMod++) {

                                        int noModd = innerComp[noComp3D][noFixed1][noComp2D][noFixed2][noComp1D].embeddedModuls[noMod];

                                        cout << endl << "NO MOD : " <<  noModd;

                                        cout << endl << "innerComp[" << noComp3D << "][" << noFixed1 << "][" << noComp2D << "][" << noFixed2 << "][" << noComp1D << "].embeddedModuls[" << noMod << "] : "
                                             << boost::get(boost::vertex_name_t(), myGraph, innerComp[noComp3D][noFixed1][noComp2D][noFixed2][noComp1D].embeddedModuls[noMod]);

                                }

                        cout << endl;

                        }
                }
        }
};

 
void Order::setOuter() { 

        // ETAPE 3 DE LA PROCEDURE A DEUX DIMENSION 
	// -----------------------------------------

	std::vector<int> outerComponentsList;

	outerComponentsList.resize(boost::num_vertices(myGraph));

        std::vector<vertexDescriptor> verticesDiscoverList(boost::num_vertices(myGraph));

        // Searching strong components in the main graph 
	// ---------------------------------------------

        boost::strong_components(myGraph, &outerComponentsList[0], boost::root_map(&verticesDiscoverList[0]));

        // Updating vertices properties of the graph : set a component Id on each vertex
	// -----------------------------------------------------------------------------

        for(int i = 0; i < outerComponentsList.size(); i++) put(get(boost::vertex_affected_comp_t(), myGraph), i, outerComponentsList[i]);

        // Setting the number of components that have been detected
	// --------------------------------------------------------

        int nbComponents = *max_element(outerComponentsList.begin(), outerComponentsList.end()) + 1;

        // Resetting the size of the outer structure table : one table entry for each detected component 
	// ---------------------------------------------------------------------------------------------

	outerComp.resize(nbComponents);

        // ETAPE 5 DE LA PROCEDURE A DEUX DIMENSION 
	// ----------------------------------------

	// For each component, set the sign for the first, second  and third dimensions then register this sign in the cfcSign attribute of the component's table entry
	// ------------------------------------------------------------------------------------------------------------------------------------------------------------

	int **isImposed = new int*[nbComponents];

	int **isImposed1D = new int*[nbComponents];

        for(int i = 0; i < nbComponents; i++) isImposed[i] = new int[3];

	for(int i = 0; i < nbComponents; i++) isImposed1D[i] = new int[3];

        for(int noComponent = 0; noComponent < nbComponents; noComponent++) {

		isImposed1D[noComponent][0] = 0;
		isImposed1D[noComponent][1] = 0;
		isImposed1D[noComponent][2] = 0;

		// Setting inial imposed values (0 mean not imposed)
		// -------------------------------------------------

		isImposed[noComponent][0] = 0;
		isImposed[noComponent][1] = 0;
		isImposed[noComponent][2] = 0; 

		// Setting initial size of the cfcSign vector
		// ------------------------------------------	
	
        	outerComp[noComponent].cfcSign.resize(3);

		// Setting initial axis numbers (i -> j -> k)
		// ------------------------------------------

        	outerComp[noComponent].cfcSign[0].first = 0;
        	outerComp[noComponent].cfcSign[1].first = 1;
        	outerComp[noComponent].cfcSign[2].first = 2;
	
		// Getting the axis values (see getCommonSign comments)
		// ----------------------------------------------------

		outerComp[noComponent].cfcSign[0].second = getCommonSign(0, noComponent, myGraph);
                outerComp[noComponent].cfcSign[1].second = getCommonSign(1, noComponent, myGraph);
                outerComp[noComponent].cfcSign[2].second = getCommonSign(2, noComponent, myGraph);

	        // Detecting all possible axis combinations then setting the one corresponding to the current component 
        	// ----------------------------------------------------------------------------------------------------

		int sign0 = outerComp[noComponent].cfcSign[0].second;
		int sign1 = outerComp[noComponent].cfcSign[1].second;
		int sign2 = outerComp[noComponent].cfcSign[2].second;

		// If there is no connection between the vertices of the component (ie single modul without reflexive edge)
                // --------------------------------------------------------------------------------------------------------

		bool isConnected = true;

		if(sign0 == 3 && sign1 == 3 && sign2 == 3) { 
		
			outerComp[noComponent].cfcSign[0].second = 0;
			outerComp[noComponent].cfcSign[1].second = 0;
			outerComp[noComponent].cfcSign[2].second = 0;
	
			isConnected = false;
		} 

		if(isConnected) {

			// Detect the type of configuration of the component accordignly to its defined and not defined axis 
			// -------------------------------------------------------------------------------------------------

			int noConf = -1;
			
			if(sign0 != 2 && sign1 != 2 && sign2 != 2) noConf = 0; 		// i -> j -> k (j and k free) 
			if(sign0 != 2 && sign1 != 2 && sign2 == 2) noConf = 1; 		// i -> j -> k (j free) 
			if(sign0 != 2 && sign1 == 2 && sign2 == 2) noConf = 2; 		// i -> j -> k
		 	if(sign0 != 2 && sign1 == 2 && sign2 != 2) noConf = 3;		// i -> k -> j (k free)
                	if(sign0 == 2 && sign1 != 2 && sign2 != 2) noConf = 4;          // j -> i -> k (i free)
			if(sign0 == 2 && sign1 != 2 && sign2 == 2) noConf = 5; 		// j -> i -> k
			if(sign0 == 2 && sign1 == 2 && sign2 != 2) noConf = 6; 		// k -> i -> j
	
			// Detect the errors of the component accordignly to its defined and not defined axis and to the null values
			// ---------------------------------------------------------------------------------------------------------
			
			if(sign0 == 2 && sign1 == 2 && sign2 == 2) noConf = 7; 		// Error : there is no common value in any dimension  
			if(sign0 == 2 && sign1 == 2 && sign2 == 0) noConf = 7;          // Error : there is no common value in any dimension
			if(sign0 == 2 && sign1 == 0 && sign2 == 2) noConf = 7;          // Error : there is no common value in any dimension
			if(sign0 == 0 && sign1 == 2 && sign2 == 2) noConf = 7;          // Error : there is no common value in any dimension
			if(sign0 == 2 && sign1 == 0 && sign2 == 0) noConf = 7;          // Error : there is no common value in any dimension
			if(sign0 == 0 && sign1 == 2 && sign2 == 0) noConf = 7;          // Error : there is no common value in any dimension
			if(sign0 == 0 && sign1 == 0 && sign2 == 2) noConf = 7;          // Error : there is no common value in any dimension
			if(sign0 == 0 && sign1 == 0 && sign2 == 0) noConf = 8; 		// Error : cyclic dependency 

			// Looking at combinations to set imposed values for the second axis (see "getImposed" comments)
			// ---------------------------------------------------------------------------------------------

			if(noConf == 0 || noConf == 1 || noConf == 2) isImposed[noComponent][1] = getImposed(0, 1, noComponent, myGraph);
			if(noConf == 3) isImposed[noComponent][2] = getImposed(0, 2, noComponent, myGraph);
			if(noConf == 4) isImposed[noComponent][2] = getImposed(1, 2, noComponent, myGraph);
			if(noConf == 5) isImposed[noComponent][1] = getImposed(1, 0, noComponent, myGraph);
			if(noConf == 6) isImposed[noComponent][2] = getImposed(2, 0, noComponent, myGraph);

			// Set imposed values for the third axis (see "getImposed" comments)
                	// ----------------------------------------------------------------

                	isImposed1D[noComponent][0] = getImposed(0, noComponent, myGraph);
                	isImposed1D[noComponent][1] = getImposed(1, noComponent, myGraph);
                	isImposed1D[noComponent][2] = getImposed(2, noComponent, myGraph);

			// Looking at combinations to : 1 - sort the axis (number and value) inside the "cfcSign" array
			//			        2 - reset axis value if this one is free
			//			        3 - reset axis value if this one is imposed
			// --------------------------------------------------------------------------------------------

			switch(noConf) {

			case 0:		

			outerComp[noComponent].cfcSign[1].second = 0;		// set j to free
			outerComp[noComponent].cfcSign[2].second = 0;		// set k to free

			if(isImposed[noComponent][0] != 0) outerComp[noComponent].cfcSign[0].second = isImposed[noComponent][0]; 
        		if(isImposed[noComponent][1] != 0) outerComp[noComponent].cfcSign[1].second = isImposed[noComponent][1];
			if(isImposed[noComponent][2] != 0) outerComp[noComponent].cfcSign[2].second = isImposed[noComponent][2];

			if(isImposed1D[noComponent][0] != 0) outerComp[noComponent].cfcSign[0].second = isImposed1D[noComponent][0];
			if(isImposed1D[noComponent][1] != 0) outerComp[noComponent].cfcSign[1].second = isImposed1D[noComponent][1];
			if(isImposed1D[noComponent][2] != 0) outerComp[noComponent].cfcSign[2].second = isImposed1D[noComponent][2];

			// If the third axis does not have a common sign 
			// If the second axis have a common sign 
			// If it is not needed to decompose the third axis dimension  (see "isToDecompose" comments)
			// Then set the axis value to 3 (means no common sign but no decomposition) 

			if(outerComp[noComponent].cfcSign[2].second == 2)  	
			if(outerComp[noComponent].cfcSign[1].second != 2) 	
			if(!isToDecompose(2, myGraph))
		
			outerComp[noComponent].cfcSign[2].second = 3;			

			break;

			case 1:

			outerComp[noComponent].cfcSign[1].second = 0;

			if(isImposed[noComponent][0] != 0) outerComp[noComponent].cfcSign[0].second = isImposed[noComponent][0];
			if(isImposed[noComponent][1] != 0) outerComp[noComponent].cfcSign[1].second = isImposed[noComponent][1];
			if(isImposed[noComponent][2] != 0) outerComp[noComponent].cfcSign[2].second = isImposed[noComponent][2];

			if(isImposed1D[noComponent][0] != 0) outerComp[noComponent].cfcSign[0].second = isImposed1D[noComponent][0];
			if(isImposed1D[noComponent][1] != 0) outerComp[noComponent].cfcSign[1].second = isImposed1D[noComponent][1];
			if(isImposed1D[noComponent][2] != 0) outerComp[noComponent].cfcSign[2].second = isImposed1D[noComponent][2];

			if(outerComp[noComponent].cfcSign[2].second == 2)
			if(outerComp[noComponent].cfcSign[1].second != 2) 
			if(!isToDecompose(2, myGraph))

		 	outerComp[noComponent].cfcSign[2].second = 3;

			break;

			case 2:

			if(isImposed[noComponent][0] != 0) outerComp[noComponent].cfcSign[0].second = isImposed[noComponent][0];
 	       		if(isImposed[noComponent][1] != 0) outerComp[noComponent].cfcSign[1].second = isImposed[noComponent][1];
        		if(isImposed[noComponent][2] != 0) outerComp[noComponent].cfcSign[2].second = isImposed[noComponent][2];

			if(isImposed1D[noComponent][0] != 0) outerComp[noComponent].cfcSign[0].second = isImposed1D[noComponent][0];
                	if(isImposed1D[noComponent][1] != 0) outerComp[noComponent].cfcSign[1].second = isImposed1D[noComponent][1];
                	if(isImposed1D[noComponent][2] != 0) outerComp[noComponent].cfcSign[2].second = isImposed1D[noComponent][2];

			if(outerComp[noComponent].cfcSign[2].second == 2)
			if(outerComp[noComponent].cfcSign[1].second != 2) 
			if(!isToDecompose(2, myGraph))
		
			outerComp[noComponent].cfcSign[2].second = 3;

			break;

			case 3:

			outerComp[noComponent].cfcSign[1].second = 0;
			outerComp[noComponent].cfcSign[2].second = 2;
			outerComp[noComponent].cfcSign[1].first = 2;
			outerComp[noComponent].cfcSign[2].first = 1;

			if(isImposed[noComponent][0] != 0) outerComp[noComponent].cfcSign[0].second = isImposed[noComponent][0];
			if(isImposed[noComponent][2] != 0) outerComp[noComponent].cfcSign[1].second = isImposed[noComponent][2];
			if(isImposed[noComponent][1] != 0) outerComp[noComponent].cfcSign[2].second = isImposed[noComponent][1];

			if(isImposed1D[noComponent][0] != 0) outerComp[noComponent].cfcSign[0].second = isImposed1D[noComponent][0];
			if(isImposed1D[noComponent][2] != 0) outerComp[noComponent].cfcSign[1].second = isImposed1D[noComponent][2];
			if(isImposed1D[noComponent][1] != 0) outerComp[noComponent].cfcSign[2].second = isImposed1D[noComponent][1];  

			if(outerComp[noComponent].cfcSign[1].second == 2)
			if(outerComp[noComponent].cfcSign[2].second != 2) 
			if(!isToDecompose(1, myGraph))
	
			outerComp[noComponent].cfcSign[1].second = 3;

			break;

			case 4:

                	outerComp[noComponent].cfcSign[0].second = outerComp[noComponent].cfcSign[1].second;
                	outerComp[noComponent].cfcSign[1].second = 0;

                	outerComp[noComponent].cfcSign[2].second = 2;
                	outerComp[noComponent].cfcSign[0].first = 1;
                	outerComp[noComponent].cfcSign[1].first = 0;
                	outerComp[noComponent].cfcSign[2].first = 2;

                	if(isImposed[noComponent][1] != 0) outerComp[noComponent].cfcSign[0].second = isImposed[noComponent][1];
                	if(isImposed[noComponent][0] != 0) outerComp[noComponent].cfcSign[1].second = isImposed[noComponent][0];
                	if(isImposed[noComponent][2] != 0) outerComp[noComponent].cfcSign[2].second = isImposed[noComponent][2];

                	if(isImposed1D[noComponent][1] != 0) outerComp[noComponent].cfcSign[0].second = isImposed1D[noComponent][1];
                	if(isImposed1D[noComponent][0] != 0) outerComp[noComponent].cfcSign[1].second = isImposed1D[noComponent][0];
                	if(isImposed1D[noComponent][2] != 0) outerComp[noComponent].cfcSign[2].second = isImposed1D[noComponent][2];

                	if(outerComp[noComponent].cfcSign[0].second == 2)
                	if(outerComp[noComponent].cfcSign[2].second != 2)
                	if(!isToDecompose(0, myGraph))

                	outerComp[noComponent].cfcSign[0].second = 3;

                	break;

			case 5:

			outerComp[noComponent].cfcSign[0].second = outerComp[noComponent].cfcSign[1].second;
			outerComp[noComponent].cfcSign[1].second = 2;
			outerComp[noComponent].cfcSign[2].second = 2;
			outerComp[noComponent].cfcSign[0].first = 1;
			outerComp[noComponent].cfcSign[1].first = 0;
			outerComp[noComponent].cfcSign[2].first = 2;

			if(isImposed[noComponent][1] != 0) outerComp[noComponent].cfcSign[0].second = isImposed[noComponent][1];
			if(isImposed[noComponent][0] != 0) outerComp[noComponent].cfcSign[1].second = isImposed[noComponent][0];
			if(isImposed[noComponent][2] != 0) outerComp[noComponent].cfcSign[2].second = isImposed[noComponent][2];

			if(isImposed1D[noComponent][1] != 0) outerComp[noComponent].cfcSign[0].second = isImposed1D[noComponent][1];
			if(isImposed1D[noComponent][0] != 0) outerComp[noComponent].cfcSign[1].second = isImposed1D[noComponent][0];
			if(isImposed1D[noComponent][2] != 0) outerComp[noComponent].cfcSign[2].second = isImposed1D[noComponent][2];

			if(outerComp[noComponent].cfcSign[2].second == 2)
			if(outerComp[noComponent].cfcSign[0].second != 2)
			if(!isToDecompose(2, myGraph))

			outerComp[noComponent].cfcSign[2].second = 3;

		
			break;

			case 6:
	
			outerComp[noComponent].cfcSign[0].second = outerComp[noComponent].cfcSign[2].second;
			outerComp[noComponent].cfcSign[1].second = 2;
			outerComp[noComponent].cfcSign[2].second = 2;
			outerComp[noComponent].cfcSign[0].first = 2;
			outerComp[noComponent].cfcSign[1].first = 0;
			outerComp[noComponent].cfcSign[2].first = 1;

			if(isImposed[noComponent][2] != 0) outerComp[noComponent].cfcSign[0].second = isImposed[noComponent][2];
			if(isImposed[noComponent][0] != 0) outerComp[noComponent].cfcSign[1].second = isImposed[noComponent][0];
			if(isImposed[noComponent][1] != 0) outerComp[noComponent].cfcSign[2].second = isImposed[noComponent][1];

			if(isImposed1D[noComponent][2] != 0) outerComp[noComponent].cfcSign[0].second = isImposed1D[noComponent][2];
			if(isImposed1D[noComponent][0] != 0) outerComp[noComponent].cfcSign[1].second = isImposed1D[noComponent][0];
			if(isImposed1D[noComponent][1] != 0) outerComp[noComponent].cfcSign[2].second = isImposed1D[noComponent][1];

			if(outerComp[noComponent].cfcSign[1].second == 2)	
			if(outerComp[noComponent].cfcSign[0].second != 2) 
			if(!isToDecompose(1, myGraph))

			outerComp[noComponent].cfcSign[1].second = 3;

			break;

			// ETAPE 4 DE LA PROCEDURE A DEUX DIMENSION (cf. docs)
			// ---------------------------------------------------

			case 7: 

			cout << endl << "Error : no common value in the component including moduls ";
			for(tie(vertexIterOne, vertexIterTwo) = vertices(myGraph); vertexIterOne != vertexIterTwo; ++vertexIterOne) 
				if(get(boost::vertex_affected_comp_t(), myGraph, *vertexIterOne) == noComponent) 
					cout << get(boost::vertex_name_t(), myGraph, *vertexIterOne) << " ";
			// SORTIE PROGRAMME

			break;

			case 8: 

			cout << endl << "Error : cyclic dependency in the component including moduls ";
				for(tie(vertexIterOne, vertexIterTwo) = vertices(myGraph); vertexIterOne != vertexIterTwo; ++vertexIterOne) 
					if(get(boost::vertex_affected_comp_t(), myGraph, *vertexIterOne) == noComponent) 
						cout << get(boost::vertex_name_t(), myGraph, *vertexIterOne) << " ";
			// SORTIE PROGRAMME
			
			break;	

			}
	
		}

	}


        // ETAPE 6 DE LA PROCEDURE A DEUX DIMENSION 
	// ----------------------------------------

	myGraphType cfcGraph;

	// Create a graph whose vertices are the components (reduced graph)	
	// ----------------------------------------------------------------

	reduceGraph(myGraph, cfcGraph, nbComponents); 

        // ETAPE 7 DE LA PROCEDURE A DEUX DIMENSION 
	// ----------------------------------------

	std::deque<int> topoOrder;

	if(boost::num_vertices(cfcGraph) > 1) { 


        // Perform a topological sort on the reduced graph
	// -----------------------------------------------

        boost::topological_sort(cfcGraph, std::front_inserter(topoOrder), boost::vertex_index_map(boost::identity_property_map()));

        // ETAPE 8 DE LA PROCEDURE A DEUX DIMENSION 
	// ----------------------------------------

        // Sort the outer structure table elements (main graph components) accordignly to the sorting performed on the reduced graph 
	// -------------------------------------------------------------------------------------------------------------------------

        	int **sortedIsImposed = new int*[nbComponents];

        	for(int i = 0; i < nbComponents; i++) sortedIsImposed[i] = new int[3];

        	vector<cfcStruct> sortedOuterCfcStructTable;

        	for(int noComponent = 0; noComponent < topoOrder.size(); noComponent++) {

        		sortedOuterCfcStructTable.push_back(outerComp[topoOrder[noComponent]]);

			sortedIsImposed[noComponent][0] = isImposed[topoOrder[noComponent]][0];
        		sortedIsImposed[noComponent][1] = isImposed[topoOrder[noComponent]][1];
        		sortedIsImposed[noComponent][2] = isImposed[topoOrder[noComponent]][2];

        	}

        	outerComp.clear();

        	for(int noComponent = 0; noComponent < topoOrder.size(); noComponent++) {

			isImposed[noComponent][0] = sortedIsImposed[noComponent][0];
        		isImposed[noComponent][1] = sortedIsImposed[noComponent][1];
        		isImposed[noComponent][2] = sortedIsImposed[noComponent][2];

        		outerComp.push_back(sortedOuterCfcStructTable[noComponent]);

        	}

	 }  else { topoOrder[0] = 0; } // Added to avoid errors if there is just one vertex 


	// Set de middle component (2D) array first dimension accordignly to the number of 3D components 
	// ---------------------------------------------------------------------------------------------

	middleComp = new cfcStructTableType*[outerComp.size()];

	// Set de inner component (1D) array first dimension accordignly to the number of 3D components
	// ---------------------------------------------------------------------------------------------

	innerComp = new cfcStructTableType***[outerComp.size()];

	// Set the middle component (2D) array second dimension to 3 (one by first fixed dimension)
	// ----------------------------------------------------------------------------------------

	for(int noComp3D = 0; noComp3D <  outerComp.size(); noComp3D++) middleComp[noComp3D]  = new cfcStructTableType[3];

	// Set the inner component (1D) array second dimension to 3 (one by first fixed dimension)
	// ---------------------------------------------------------------------------------------

 	for(int noComp3D = 0; noComp3D <  outerComp.size(); noComp3D++) innerComp[noComp3D]  = new cfcStructTableType**[3];

	// For each 3D component 
	// ---------------------

	for(int noComp = 0; noComp <  outerComp.size(); noComp++) {

	// Create a graph that just contain vertices and edges included in the current 3D component (ie extract the current 3D component from the main graph)
	// --------------------------------------------------------------------------------------------------------------------------------------------------

        	myGraphType outerGraph;

        	copyGraph(myGraph, outerGraph);


        	// Create a graph that just keep the current 3D component egdes and vertices 
		// -------------------------------------------------------------------------

		keepComponent(topoOrder[noComp], outerGraph);


		// Launch the functions for analizing 2D components included inside the current 3D component (see "setMiddle" comments)
		// --------------------------------------------------------------------------------------------------------------------

        	if(boost::num_vertices(outerGraph) > 0) {

        	setMiddle(0, 1, 2, outerGraph, noComp);
        	setMiddle(1, 0, 2, outerGraph, noComp);
        	setMiddle(2, 0, 1, outerGraph, noComp);

        	}

        }      // NEXT 3D COMPONENT 

};





void Order::keepComponent(int noComp, myGraphType& componentGraph) {

        typedef boost::graph_traits<myGraphType>::edge_iterator edgeIterType;

        std::vector<edgeIterType> edgeIterVector;

        // Insert all edges of the graph inside a vector of edge
        // -----------------------------------------------------

        for(tie(edgeIterOne, edgeIterTwo) = edges(componentGraph); edgeIterOne != edgeIterTwo; ++edgeIterOne) edgeIterVector.push_back(edgeIterOne);

        // Reverse the vector to avoid a mess in the indexes when the edges are deleted
        // ----------------------------------------------------------------------------

        reverse(edgeIterVector.begin(), edgeIterVector.end());

        // For all edges embeded in the vector
        // -----------------------------------

        for(int i = 0; i < edgeIterVector.size(); i++)

                // If the edge does not connect two nodes of the component "noComp" given as parameter (ie. if the edge is not fully integrated in the comoponent)
                // -----------------------------------------------------------------------------------------------------------------------------------------------

                if( get(boost::vertex_affected_comp_t(), componentGraph, boost::source(*(edgeIterVector[i]), componentGraph)) != noComp ||

                    get(boost::vertex_affected_comp_t(), componentGraph, boost::target(*(edgeIterVector[i]), componentGraph)) != noComp )


                                // Remove the edge for keeping just edges of the relevant component
                                // ----------------------------------------------------------------

                                boost::remove_edge(*(edgeIterVector[i]), componentGraph);

        typedef boost::graph_traits<myGraphType>::vertex_iterator vertexIterType;

        std::vector<vertexIterType> vertexIterVector;

        // Insert all vertices of the graph inside a vector of vertex
        // ----------------------------------------------------------

        for(tie(vertexIterOne, vertexIterTwo) = vertices(componentGraph); vertexIterOne != vertexIterTwo; ++vertexIterOne) vertexIterVector.push_back(vertexIterOne);

        // Reverse the vector to avoid a mess in the indexes when the vertices are deleted
        // ----------------------------------------------------------------------------

        reverse(vertexIterVector.begin(), vertexIterVector.end());

        // For all vertices embeded in the vector
        // -----------------------------------

        for(int i = 0; i < vertexIterVector.size(); i++)

                // If the vertex is not a part of the component "noComp" given as parameter
                // ------------------------------------------------------------------------

                if(get(boost::vertex_affected_comp_t(), componentGraph, *(vertexIterVector[i])) != noComp)

                        // Remove the vertex for keeping just vertices of the relevant component
                        // ---------------------------------------------------------------------

                        boost::remove_vertex(*(vertexIterVector[i]), componentGraph);

        };


void Order::showMiddle(int noFixed1) { 

		cout << endl << "showMiddle(" << noFixed1 << ")";

		for(int noComp3D = 0; noComp3D < outerComp.size(); noComp3D++) {

                	for(int noComp2D = 0; noComp2D < middleComp[noComp3D][noFixed1].size(); noComp2D++) {

                                cout << endl    << "middleComp[" << noComp3D << "][" << noFixed1 << "][" << noComp2D << "].cfcSign[0].first : " 
                                                <<  middleComp[noComp3D][noFixed1][noComp2D].cfcSign[0].first << " , second : " <<  middleComp[noComp3D][noFixed1][noComp2D].cfcSign[0].second;

                                cout << endl    << "middleComp[" << noComp3D << "][" << noFixed1 << "][" << noComp2D << "].cfcSign[1].first : "
                                                <<  middleComp[noComp3D][noFixed1][noComp2D].cfcSign[1].first << " , second : " <<  middleComp[noComp3D][noFixed1][noComp2D].cfcSign[1].second;

				cout << endl; 

				for(int noMod = 0; noMod < middleComp[noComp3D][noFixed1][noComp2D].embeddedModuls.size(); noMod++) {

                                	cout << endl << "middleComp[" << noComp3D << "][" << noFixed1 << "][" << noComp2D << "].EmbeddedModuls[" << noMod << "] : "
                                                     << get(boost::vertex_name_t(), myGraph, middleComp[noComp3D][noFixed1][noComp2D].embeddedModuls[noMod]) << " ==> NoMod : " <<  middleComp[noComp3D][noFixed1][noComp2D].embeddedModuls[noMod];

				}

				cout << endl;
			}
		} 
	};





int Order::getCommonSign(int noAxis,  myGraphType& fixedGraph) {

// Initial value set to zero (which means free sign)
// -------------------------------------------------

int recVal = 0;

        // For all edges of the graph 
        // --------------------------

        for(tie(edgeIterOne, edgeIterTwo) = edges(fixedGraph); edgeIterOne != edgeIterTwo; ++edgeIterOne) {

                // Get informations on the sign for the dimensions i, j and k
                // -----------------------------------------------------------

                int val0 = get(boost::edge_weight_d1_t(), fixedGraph, *edgeIterOne);
                int val1 = get(boost::edge_weight_d2_t(), fixedGraph, *edgeIterOne);
                int val2 = get(boost::edge_weight_d3_t(), fixedGraph, *edgeIterOne);

                // If the recorded value recVal is not equal to 2 (ie. if it is equal to 2, there is no need to go further in the sign analyzis)
                // -----------------------------------------------------------------------------------------------------------------------------

                if(recVal != 2) {

                        // The sign analyzed is this of the dimension "noAxis" given as parameter 
                        // ----------------------------------------------------------------------

                        switch(noAxis) {

                                // If the current value is not null and recVal + the current value is equal to zero, it means there is a couple -1 an +1 and then no common sign
                                // ---------------------------------------------------------------------------------------------------------------------------------------------

                                case 0: if(val0 != 0) { if(recVal + val0 == 0) { recVal = 2; } else { recVal = val0; } } break;
                                case 1: if(val1 != 0) { if(recVal + val1 == 0) { recVal = 2; } else { recVal = val1; } } break;
                                case 2: if(val2 != 0) { if(recVal + val2 == 0) { recVal = 2; } else { recVal = val2; } } break;
                        }
                }
        }

return(recVal);

};




void Order::copyGraph( myGraphType& upperGraph, myGraphType& outGraph) {

// Create a new graph "outGraph" with one vertex for each vertex of the graph that have to be copied "upperGraph"
// --------------------------------------------------------------------------------------------------------------

for(int noVertex = 0; noVertex < boost::num_vertices(upperGraph); noVertex++) add_vertex(outGraph);

// Add one edge for each edge of the graph that have to be copied (the vertices are created with the same indexes than in the original graph and then the edges connect the right vertices)
// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

for(tie(edgeIterOne, edgeIterTwo) = edges(upperGraph); edgeIterOne != edgeIterTwo; ++edgeIterOne) add_edge(boost::source(*edgeIterOne, upperGraph), boost::target(*edgeIterOne, upperGraph), outGraph);

// For all vertices
// ----------------

for(tie(vertexIterOne, vertexIterTwo) = vertices(upperGraph); vertexIterOne != vertexIterTwo; ++vertexIterOne) {

        // Copy the properties from the original graph into the target graph
        // -----------------------------------------------------------------

        put(get(boost::vertex_name_t(),          outGraph),     *vertexIterOne,         get(boost::vertex_name_t(),             upperGraph, *vertexIterOne));
        put(get(boost::vertex_topo_order_t(),    outGraph),     *vertexIterOne,         get(boost::vertex_topo_order_t(),       upperGraph, *vertexIterOne));
        put(get(boost::vertex_affected_comp_t(), outGraph),     *vertexIterOne,         get(boost::vertex_affected_comp_t(),    upperGraph, *vertexIterOne));
        put(get(boost::vertex_original_id_t(),   outGraph),     *vertexIterOne,         get(boost::vertex_original_id_t(),      upperGraph, *vertexIterOne));

        }


/*      IMPOSSIBLE DE REALISER LA MEME CHOSE QU'AVEC LES SOMMETS ==> BUG BGL MANIFESTE !!!!!!!

for(tie(edgeIterOne, edgeIterTwo) = edges(upperGraph); edgeIterOne != edgeIterTwo; ++edgeIterOne) {

        put(get(boost::edge_weight_d1_t(), outGraph), *edgeIterOne, get(boost::edge_weight_d1_t(), upperGraph, *edgeIterOne));
        put(get(boost::edge_weight_d2_t(), outGraph), *edgeIterOne, get(boost::edge_weight_d2_t(), upperGraph, *edgeIterOne));
        put(get(boost::edge_weight_d3_t(), outGraph), *edgeIterOne, get(boost::edge_weight_d3_t(), upperGraph, *edgeIterOne));

        }
*/

// As it is not possible to populate the edges properties the same way than the vertices properties, this operation is made in two steps : data structures are populated
// from the source graph then target graph properties are populated from the structures 

// Create one vector by edge property that have to be copied in the new graph 
// --------------------------------------------------------------------------

std::vector<int> dim_1;
std::vector<int> dim_2;
std::vector<int> dim_3;

// For all edges in the original graph
// -----------------------------------

for(tie(edgeIterOne, edgeIterTwo) = edges(upperGraph); edgeIterOne != edgeIterTwo; ++edgeIterOne) {

        // Insert the properties into the dedicated vectors 
        // ------------------------------------------------

        dim_1.push_back(get(boost::edge_weight_d1_t(), upperGraph, *edgeIterOne));
        dim_2.push_back(get(boost::edge_weight_d2_t(), upperGraph, *edgeIterOne));
        dim_3.push_back(get(boost::edge_weight_d3_t(), upperGraph, *edgeIterOne));

        }

int cmptArc = 0;

// For all edges in the target graph
// ---------------------------------

for(tie(edgeIterOne, edgeIterTwo) = edges(outGraph); edgeIterOne != edgeIterTwo; ++edgeIterOne) {

        // Populate the properties from the dedicated vector
        // -------------------------------------------------

        put(get(boost::edge_weight_d1_t(), outGraph), *edgeIterOne, dim_1[cmptArc]);
        put(get(boost::edge_weight_d2_t(), outGraph), *edgeIterOne, dim_2[cmptArc]);
        put(get(boost::edge_weight_d3_t(), outGraph), *edgeIterOne, dim_3[cmptArc]);

        cmptArc++;

        }

};


void Order::showGraph( myGraphType& currentGraph) {

        cout << endl << "showGraph : " <<  endl << endl;

        int cmptMod = 0;

        // For all vertex embedded in the current graph 
        // --------------------------------------------

        for(tie(vertexIterOne, vertexIterTwo) = vertices(currentGraph); vertexIterOne != vertexIterTwo; ++vertexIterOne) {

                // Display all relevant informations on the vertex (ie the modul) 
                // --------------------------------------------------------------

                cout    << endl << "Module " << cmptMod << ":\t"

                        // Name of the modul as it appears in the description file 
                        // -------------------------------------------------------

                        << "Nom : " << get(boost::vertex_name_t(), myGraph, get(boost::vertex_original_id_t(), currentGraph, *vertexIterOne))

                        // Order affected to the modul in the current graph (iformation coming from the topological sort)
                        // ----------------------------------------------------------------------------------------------

                        << ", No ordre : "      << get(boost::vertex_topo_order_t(), currentGraph, *vertexIterOne)

                        // Component in which the modul is integrated (infromation coming from the strong component search) 
                        // ------------------------------------------------------------------------------------------------

                        << ", No Composante : " << get(boost::vertex_affected_comp_t(), currentGraph, *vertexIterOne)

                        // Index that is affected to the modul in the main graph (as vertices are automatically indexed by boost, it is very important to keep this information inside the sub-graphs) 
                        // ---------------------------------------------------------------------------------------------------------------------------------------------------------------------------

                        << ", No Id graph original : " << get(boost::vertex_original_id_t(), currentGraph, *vertexIterOne);

                cmptMod++;
        }


        int cmptArc = 0;

        cout << endl << endl;

        // For all edge embedded in the current graph 
        // ------------------------------------------

        for(tie(edgeIterOne, edgeIterTwo) = edges(currentGraph); edgeIterOne != edgeIterTwo; ++edgeIterOne) {

                // Display all relevant informations on the edge (ie connection between two moduls) 
                // --------------------------------------------------------------------------------

                cout    << endl << "Arc " << cmptArc << " :\t"

                        // Name of the source modul of the connection 
                        // ------------------------------------------

                        << get(boost::vertex_name_t(), currentGraph,  boost::source(*edgeIterOne, currentGraph))

                        // Sign of the dimension i of the connection 
                        // -----------------------------------------

                        << " ( " << get(boost::edge_weight_d1_t(), currentGraph, *edgeIterOne)

                        // Sign of the dimension j of the connection 
                        // -----------------------------------------

                        << ", "  << get(boost::edge_weight_d2_t(), currentGraph, *edgeIterOne)

                        // Sign of the dimension k of the connection 
                        // -----------------------------------------

                        << ", "  << get(boost::edge_weight_d3_t(), currentGraph, *edgeIterOne)

                        // Name of the tagret modul of the connection 
                        // ------------------------------------------

                        <<") ==> " << get(boost::vertex_name_t(), currentGraph,  boost::target(*edgeIterOne, currentGraph));
        cmptArc++;
         
        }
};

                     
 
void Order::readParam(int noAxis, int valAxis, int noParam) {

// Static value that record the last parameter level that have been inserted (ie parameter position in the order statement)
// ------------------------------------------------------------------------------------------------------------------------

static int lastParam;

// The function have a special behaviour if noParam have a null value
// ------------------------------------------------------------------

if(noParam > 0) {

        // Record the difference between the current parameter level and the former parameter level
        // ----------------------------------------------------------------------------------------

        int deltaParam = lastParam - noParam;

        // Reset the lastParam static value with the current parameter level (for the next function call) 
        // ----------------------------------------------------------------------------------------------

        lastParam = noParam;

        // Accordignly to the difference of levels, the number of "forder" tokens to close the order statement will be different 
        // ---------------------------------------------------------------------------------------------------------------------

        switch(deltaParam) {

                        // If there is no difference between the last level and the current one, then one order have to be closed (those of the current level)
                        // -----------------------------------------------------------------------------------------------------------------------------------

                        case 0:

                                // If the current level is the third one, it means the order have to be closed with two tabulation offsets 
                                // -------------------------------------------------------------------------------------------------------

                                if(noParam == 3) cout << endl << endl << "\t\tforder";

                                // If the current level is the second one, it means the order have to be closed with one tabulation offset
                                // -------------------------------------------------------------------------------------------------------

                                if(noParam == 2) cout << endl << endl << "\tforder";

                                // If the current level is the first one, it means the order have to be closed without tabulation offset 
                                // -----------------------------------------------------------------------------------------------------

                                if(noParam == 1) cout << endl << endl << "forder";
                        break;

                        // If the last parameter is one level higer than the current one, then two parameters have to be closed  (one for the current level and one for the higher)
                        // --------------------------------------------------------------------------------------------------------------------------------------------------------

                        case 1:

                                // If the current parameter is the second one, the order statement have to be closed with two tabulation offset then one tabulation offset 
                                // ---------------------------------------------------------------------------------------------------------------------------------------

                                if(noParam == 2) cout << endl << endl << "\t\tforder" << endl << "\tforder";

                                // If the current parameter is the first one, the order statement  have to be closed with one tabulation offset then no offset
                                // ---------------------------------------------------------------------------------------------------------------------------

                                if(noParam == 1) cout << endl << endl << "\tforder" << endl << "forder";
                        break;

                        // If the last parameter is two levels higer than the current one, then three parameters have to be closed  (one for the current level and one for each higer)
                        // -----------------------------------------------------------------------------------------------------------------------------------------------------------

                        case 2:

                                // The order statement  have to be closed with two tabulations (higer level), one tabulation (middle level) and no tabulation (lower level)
                                // ----------------------------------------------------------------------------------------------------------------------------------------

                                cout << endl << endl << "\t\tforder" << endl << endl << "\tforder" << endl << "forder";

                        break;

                        }


        // Set up tabulation offsets for displaying moduls after the order parameters 
        // --------------------------------------------------------------------------

        switch(noParam) {

                        // If the parameter is the first, then no offset is needed 
                        // -------------------------------------------------------

                        case 1: cout << endl << endl; break;

                        // If the parameter is the second, then one offset is needed
                        // ---------------------------------------------------------

                        case 2: cout << endl << endl << "\t"; break;

                        // If the parameter is the third, then two offsets are needed
                        // ----------------------------------------------------------

                        case 3: cout << endl << endl << "\t\t"; break;
                        }

        // Set up the parameter sign accordignly to the axis value
        // -------------------------------------------------------

        switch(valAxis) {

                        // If the axis value is set to 0 (null values on each edges), then the dimension can be crossed both way (free)
                        // ------------------------------------------------------------------------------------------------------------

                        case  0: cout << "order " << "\t" << "Y." << noAxis + 1; break;

                        // If the axis value is set to 3 (no edges), then the dimension can be crossed both way (free)
                        // -------------------------------------------------------------------------------------------

                        case  3: cout << "order " << "\t" << "Y." << noAxis + 1; break;

                        // If the axis value is set to -1, then the dimension must be crossed increasingly 
                        // -------------------------------------------------------------------------------

                        case -1: cout << "order " << "\t" << "YA" << noAxis + 1; break;

                        // If the axis value is set to +1, then the dimension must be crossed decreasingly
                        // -------------------------------------------------------------------------------

                        case  1: cout << "order " << "\t" << "YB" << noAxis + 1; break;

                        // If the axis value is set to 2 (no common value), then the dimension can be crossed both way (free)
                        // --------------------------------------------------------------------------------------------------

                        case  2: cout << "order " << "\t" << "Y." << noAxis + 1; break;         // A VOIR !!!!
                        }
}

// If the function is called with a null value for noParam, then tho order statement must be closed 
// ------------------------------------------------------------------------------------------------

else {

        // If the last parameter was the third in the order statement, then three parameters have to be closed (one for each open parameter)
        // ---------------------------------------------------------------------------------------------------------------------------------

        if(lastParam == 3) cout << endl << endl << "\t\tforder" << endl << endl << "\tforder" << endl << "forder" << endl << endl;

        // If the last parameter was the second in the order statement, then two parameters have to be closed (one for each open parameter)
        // --------------------------------------------------------------------------------------------------------------------------------

        if(lastParam == 2) cout << endl << endl << "\tforder" << endl << "forder" << endl << endl;

        // If the last parameter was the first in the order statement, then one parameter have to be closed 
        // ------------------------------------------------------------------------------------------------

        if(lastParam == 1) cout << endl << endl << "forder" << endl << endl;
}


};


void Order::setInner(int noComp3D, int noFixed1, int noComp2D, int noFixed2, int freeDim, myGraphType& upperGraph) {

	myGraphType currentGraph;

        copyGraph(upperGraph, currentGraph);

        fixGraph(currentGraph, noFixed2);

        // *****************************************************
        // ****                                           ******
        // ****  ETAPE 1 DE LA PROCEDURE A UNE DIMENSION  ******
        // ****                                           ******
        // *****************************************************

        // Create data structures used by the component search algorithm
	// -------------------------------------------------------------

        std::vector<int> fixedComponentsList(boost::num_vertices(currentGraph));

        std::vector<vertexDescriptor> fixedVerticesDiscoverList(boost::num_vertices(currentGraph));

        // Perform the 1D component search 
	// --------------------------------

        boost::strong_components(currentGraph, &fixedComponentsList[0], boost::root_map(&fixedVerticesDiscoverList[0]));

	// Resetting all vertices'affected component 
	// -----------------------------------------

        for(tie(vertexIterOne, vertexIterTwo) = vertices(currentGraph); vertexIterOne != vertexIterTwo; ++vertexIterOne) {
        
	put(get(boost::vertex_affected_comp_t(), currentGraph), *vertexIterOne, -1);
        
	}

        // Populate the vertices property of the graph (set for each vertex its component Id) 
	// ----------------------------------------------------------------------------------

        for(int i = 0; i < fixedComponentsList.size(); i++) put(get(boost::vertex_affected_comp_t(), currentGraph), i, fixedComponentsList[i]);

        // Get the number of 1D components detected 
	// ----------------------------------------

        int nbFixedComponents = *max_element(fixedComponentsList.begin(), fixedComponentsList.end()) + 1;

        // *****************************************************
        // ****                                             ****
        // ****  ETAPE 5.1 DE LA PROCEDURE A UNE DIMENSION  ****
        // ****                                             ****
        // *****************************************************

        // update the innerComp array to allow this structure to register informations on all the 1D components detected for the dimension 
	// -------------------------------------------------------------------------------------------------------------------------------

	innerComp[noComp3D][noFixed1][noComp2D][noFixed2].resize(nbFixedComponents);

        // reset all the structure attributes 
	// ----------------------------------

	for(int noComp1D = 0; noComp1D < innerComp[noComp3D][noFixed1][noComp2D][noFixed2].size(); noComp1D++) {

		innerComp[noComp3D][noFixed1][noComp2D][noFixed2][noComp1D].cfcSign.resize(1);
		innerComp[noComp3D][noFixed1][noComp2D][noFixed2][noComp1D].cfcSign[0].second = 0;
		innerComp[noComp3D][noFixed1][noComp2D][noFixed2][noComp1D].embeddedModuls.resize(0);
        }


	// For each component, insert in the inner struct table element the embebbed moduls 
	// --------------------------------------------------------------------------------

	for(int i = 0; i < fixedComponentsList.size(); i++) {

        innerComp[noComp3D][noFixed1][noComp2D][noFixed2][fixedComponentsList[i]].embeddedModuls.push_back(boost::get(boost::vertex_original_id_t(), currentGraph, i));

	}
	

        // *******************************************************
        // ****                                             ******
        // ****  ETAPE 5.2 DE LA PROCEDURE A UNE DIMENSION  ******
        // ****                                             ******
        // *******************************************************

        // *******************************************************
        // ****                                             ******
        // ****    ETAPE 2 DE LA PROCEDURE A UNE DIMENSION  ******
        // ****                                             ******
        // *******************************************************

        // Get the common size of the component for the current dimension then store the information on the structure
	// ----------------------------------------------------------------------------------------------------------

        for(int noComp1D = 0; noComp1D < nbFixedComponents; noComp1D++) {

		innerComp[noComp3D][noFixed1][noComp2D][noFixed2][noComp1D].cfcSign[0].first = 0;

	 	innerComp[noComp3D][noFixed1][noComp2D][noFixed2][noComp1D].cfcSign[0].first  = freeDim;

		innerComp[noComp3D][noFixed1][noComp2D][noFixed2][noComp1D].cfcSign[0].second = getCommonSignInner(noFixed2, freeDim, currentGraph);

        }

        // ETAPE 3 DE LA PROCEDURE A UNE DIMENSION  
	// ---------------------------------------

        fixGraph(currentGraph, freeDim);

        // ETAPE 4 DE LA PROCEDURE A UNE DIMENSION  
	// --------------------------------------- 

        // Perform a topological sort on the graph 

        std::deque<int> topoOrder;

        boost::topological_sort(currentGraph, std::front_inserter(topoOrder), boost::vertex_index_map(boost::identity_property_map()));

        for(int i = 0; i < topoOrder.size(); i++) put(get(boost::vertex_topo_order_t(), currentGraph), topoOrder[i], i);

        // ETAPE 5.2 DE LA PROCEDURE A UNE DIMENSION 
	// -----------------------------------------

        // Sort the moduls inside the components (internal sort)  
	// -----------------------------------------------------

	for(int noComp1D = 0; noComp1D < innerComp[noComp3D][noFixed1][noComp2D][noFixed2].size(); noComp1D++) {

		for(int noMod = 0; noMod < innerComp[noComp3D][noFixed1][noComp2D][noFixed2][noComp1D].embeddedModuls.size(); noMod++) {

			for(int noMod2 = noMod + 1; noMod2 < innerComp[noComp3D][noFixed1][noComp2D][noFixed2][noComp1D].embeddedModuls.size(); noMod2++) { 

				if(get(boost::vertex_topo_order_t(), currentGraph, innerComp[noComp3D][noFixed1][noComp2D][noFixed2][noComp1D].embeddedModuls[noMod])  >
   			   	   get(boost::vertex_topo_order_t(), currentGraph , innerComp[noComp3D][noFixed1][noComp2D][noFixed2][noComp1D].embeddedModuls[noMod2])) {

					int recMod = innerComp[noComp3D][noFixed1][noComp2D][noFixed2][noComp1D].embeddedModuls[noMod];

					innerComp[noComp3D][noFixed1][noComp2D][noFixed2][noComp1D].embeddedModuls[noMod] = innerComp[noComp3D][noFixed1][noComp2D][noFixed2][noComp1D].embeddedModuls[noMod2];
		
					innerComp[noComp3D][noFixed1][noComp2D][noFixed2][noComp1D].embeddedModuls[noMod2] = recMod;
				}
			}
		}
	}

        // ETAPE 5.3 DE LA PROCEDURE A UNE DIMENSION 
	// ----------------------------------------- 

	// Create a reduced graph whose vertices are the components of the current graph
	// -----------------------------------------------------------------------------

	myGraphType cfcGraph;

        reduceGraph(currentGraph, cfcGraph, nbFixedComponents);

	if(boost::num_vertices(cfcGraph) > 1) {

        std::deque<int> reducedTopoOrder;

        // Perform a topological sort on the reduced graph
	// -----------------------------------------------

        boost::topological_sort(cfcGraph, std::front_inserter(reducedTopoOrder), boost::vertex_index_map(boost::identity_property_map()));

        // Sort the inner structure table elements accordignly to the sorting performed on the reduced graph 
	// -------------------------------------------------------------------------------------------------

        vector<cfcStruct> sortedCfcStructTable;

        for(int noComponent = 0; noComponent < reducedTopoOrder.size(); noComponent++) sortedCfcStructTable.push_back(innerComp[noComp3D][noFixed1][noComp2D][noFixed2][reducedTopoOrder[noComponent]]);

	innerComp[noComp3D][noFixed1][noComp2D][noFixed2].clear();

	for(int noComponent = 0; noComponent < reducedTopoOrder.size(); noComponent++) innerComp[noComp3D][noFixed1][noComp2D][noFixed2].push_back(sortedCfcStructTable[noComponent]);

	}

};


void Order::fixGraph(myGraphType& currentGraph, int noFixedDim)  {

        // Putting iterators inside a vector is more convenient , especially for deletions
        // -------------------------------------------------------------------------------

        typedef boost::graph_traits<myGraphType>::edge_iterator edgeIterType;

        std::vector<edgeIterType> edgeIterVector;

        // Insert all edges of the graph inside a vector of edge
        // -----------------------------------------------------

        for(tie(edgeIterOne, edgeIterTwo) = edges(currentGraph); edgeIterOne != edgeIterTwo; ++edgeIterOne) edgeIterVector.push_back(edgeIterOne);

        // Reverse the vector to avoid a mess in the indexes when the edges are deleted
        // ----------------------------------------------------------------------------

        reverse(edgeIterVector.begin(), edgeIterVector.end());

        // For all edges inside the vector
        // -------------------------------

        for(int i = 0; i < edgeIterVector.size(); i++) {

                // If the sign value for the edge is not null for the dimension "noFixedDim" given as parameter, then delete the edge
                // ------------------------------------------------------------------------------------------------------------------

                if(noFixedDim == 0) if(get(boost::edge_weight_d1_t(), currentGraph, *(edgeIterVector[i])) != 0) boost::remove_edge(*(edgeIterVector[i]), currentGraph);
                if(noFixedDim == 1) if(get(boost::edge_weight_d2_t(), currentGraph, *(edgeIterVector[i])) != 0) boost::remove_edge(*(edgeIterVector[i]), currentGraph);
                if(noFixedDim == 2) if(get(boost::edge_weight_d3_t(), currentGraph, *(edgeIterVector[i])) != 0) boost::remove_edge(*(edgeIterVector[i]), currentGraph);
        }
};



void Order::reduceGraph(myGraphType innerGraph, myGraphType& outerGraph, int nbComp) {

        // Create a graph that will contain one vertex by component
        // --------------------------------------------------------

        for(int noComp = 0; noComp < nbComp; noComp++) add_vertex(outerGraph);

        // For all edges in the graph that have to reduced 
        // -----------------------------------------------

        for(tie(edgeIterOne, edgeIterTwo) = edges(innerGraph); edgeIterOne != edgeIterTwo; ++edgeIterOne) {

                int sourceComp = get(boost::vertex_affected_comp_t(), innerGraph, boost::source(*edgeIterOne, innerGraph));

                int targetComp = get(boost::vertex_affected_comp_t(), innerGraph, boost::target(*edgeIterOne, innerGraph));

                // If the target and the source modul of the edge are not in the same component, add an edge to the reduced graph 
                // --------------------------------------------------------------------------------------------------------------

                if(sourceComp != targetComp)  add_edge(sourceComp, targetComp, outerGraph);

        }
};



void Order::setMiddle(int fixedDim, int dimOne, int dimTwo, myGraphType& currentGraph, int noComp) {


	// Create a graph that only keep zero valuated edges for the 3D fixed dim, this graph will be the 2D graph
	// -------------------------------------------------------------------------------------------------------

        myGraphType fixedGraph;

        copyGraph(currentGraph, fixedGraph); 

	fixGraph(fixedGraph, fixedDim);

        // ETAPE 3 DE LA PROCEDURE A DEUX DIMENSION 
	// ----------------------------------------

	// Create data structures used by boost to perform the strong component search 
	// ---------------------------------------------------------------------------

        std::vector<int> middleComponentsList(boost::num_vertices(fixedGraph));

        std::vector<vertexDescriptor> verticesDiscoverList(boost::num_vertices(fixedGraph));
	
	// Resetting all vertices'affected component of the 2D graph 
	// ---------------------------------------------------------

	for(tie(vertexIterOne, vertexIterTwo) = vertices(fixedGraph); vertexIterOne != vertexIterTwo; ++vertexIterOne) put(get(boost::vertex_affected_comp_t(), fixedGraph), *vertexIterOne, -1);

	// Perform a strong components search on the 2D graph 
	// --------------------------------------------------

	boost::strong_components(fixedGraph, &middleComponentsList[0], boost::root_map(&verticesDiscoverList[0]));

	// Update vertices properties of the 2D graph : set a component Id on each vertex
	// ------------------------------------------------------------------------------

	for(int i = 0; i < middleComponentsList.size(); i++) put(get(boost::vertex_affected_comp_t(), fixedGraph), i, middleComponentsList[i]);

	// Setting the number of components that have been detected
	// --------------------------------------------------------

	int nbComponents = *max_element(middleComponentsList.begin(), middleComponentsList.end()) + 1;

	// Setting the third dimension size of innerComp array
	// ---------------------------------------------------

	innerComp[noComp][fixedDim] = new cfcStructTableType*[nbComponents]; 

	// Setting the fourth dimension size of the innerComp array
	// --------------------------------------------------------

	for(int noCompD2 = 0; noCompD2 < nbComponents; noCompD2++) innerComp[noComp][fixedDim][noCompD2] = new cfcStructTableType[3];

	// ETAPE 5 DE LA PROCEDURE A DEUX DIMENSION 
	// ----------------------------------------

	// Resize the middleComp array with the size of 2D components that have been detected
	// ----------------------------------------------------------------------------------

	middleComp[noComp][fixedDim].resize(nbComponents);

	// Set up the array that will be used to register the informations about imposed values for the axis
	// -------------------------------------------------------------------------------------------------

	int ** isImposed2D = new int*[nbComponents];

	for(int i = 0; i < nbComponents; i++) isImposed2D[i] = new int[2];

	// For each component, set the sign for the first and second dimensions then register this sign in the cfcSign attribute
	// ---------------------------------------------------------------------------------------------------------------------

	for(int noComponent = 0; noComponent < nbComponents; noComponent++) {

		// Setting inial imposed values (0 mean no imposed)
		// ------------------------------------------------

		isImposed2D[noComponent][0] = 0;
		isImposed2D[noComponent][1] = 0;

		middleComp[noComp][fixedDim][noComponent].cfcSign.resize(2);

		middleComp[noComp][fixedDim][noComponent].cfcSign[0].first = dimOne;
		middleComp[noComp][fixedDim][noComponent].cfcSign[1].first = dimTwo;

		middleComp[noComp][fixedDim][noComponent].cfcSign[0].second = 0;
		middleComp[noComp][fixedDim][noComponent].cfcSign[1].second = 0;

		middleComp[noComp][fixedDim][noComponent].cfcSign[0].second = getCommonSign(dimOne, noComponent, fixedGraph); 
		middleComp[noComp][fixedDim][noComponent].cfcSign[1].second = getCommonSign(dimTwo, noComponent, fixedGraph);


		int sign0 = middleComp[noComp][fixedDim][noComponent].cfcSign[0].second;
        	int sign1 = middleComp[noComp][fixedDim][noComponent].cfcSign[1].second;

		int axis0 = dimOne;
		int axis1 = dimTwo; 

		// If there is just one vertex in the component without reflexive edge
		// -------------------------------------------------------------------

		bool isConnected = true;

                if(sign0 == 3 && sign1 == 3) {

                        middleComp[noComp][fixedDim][noComponent].cfcSign[0].second = 0;
                        middleComp[noComp][fixedDim][noComponent].cfcSign[1].second = 0;

                        isConnected = false;
                }

		if(isConnected) { 

			// Detecting all possible axis combinations 
			// ----------------------------------------

			int noConf = -1;

			// int sign0 = middleComp[noComp][fixedDim][noComponent].cfcSign[0].second;
			// int sign1 = middleComp[noComp][fixedDim][noComponent].cfcSign[1].second; 

			if(sign0 != 2 && sign1 != 2)  noConf = 0;       // axis0 -> axis1 (axis1 free)
			if(sign0 != 2 && sign1 == 2)  noConf = 1;       // axis0 -> axis1 
			if(sign0 == 2 && sign1 == 2)  noConf = 2;       // axis0 -> axis1
			if(sign0 == 2 && sign1 != 2)  noConf = 3;       // axis1 -> axis0 

			// Looking at combinations to set imposed values for the second axis 
			// -----------------------------------------------------------------

			if(noConf < 3) isImposed2D[noComponent][1] = getImposed(axis1, noComponent, fixedGraph);

			if(noConf == 3) isImposed2D[noComponent][1] = getImposed(axis0, noComponent, fixedGraph);

			// Sort the axis then set their values accordignly to the combination available for the current 2D component
			// ---------------------------------------------------------------------------------------------------------

			if(noConf == 0) { 

				middleComp[noComp][fixedDim][noComponent].cfcSign[1].second = 0;

				if(isImposed2D[noComponent][0] != 0) middleComp[noComp][fixedDim][noComponent].cfcSign[0].second = isImposed2D[noComponent][0];
				if(isImposed2D[noComponent][1] != 0) middleComp[noComp][fixedDim][noComponent].cfcSign[1].second = isImposed2D[noComponent][1]; 
			}

			if(noConf > 0 && noConf < 3) {

				if(isImposed2D[noComponent][0] != 0) middleComp[noComp][fixedDim][noComponent].cfcSign[0].second = isImposed2D[noComponent][0];
				if(isImposed2D[noComponent][1] != 0) middleComp[noComp][fixedDim][noComponent].cfcSign[1].second = isImposed2D[noComponent][1];
			}

			if(noConf == 3) { 

				int tmpVal = middleComp[noComp][fixedDim][noComponent].cfcSign[0].second;
				middleComp[noComp][fixedDim][noComponent].cfcSign[0].second = middleComp[noComp][fixedDim][noComponent].cfcSign[1].second;
				middleComp[noComp][fixedDim][noComponent].cfcSign[1].second = tmpVal;
				middleComp[noComp][fixedDim][noComponent].cfcSign[0].first = axis1;
				middleComp[noComp][fixedDim][noComponent].cfcSign[1].first = axis0;

				if(isImposed2D[noComponent][0] != 0) middleComp[noComp][fixedDim][noComponent].cfcSign[1].second = isImposed2D[noComponent][0];
				if(isImposed2D[noComponent][1] != 0) middleComp[noComp][fixedDim][noComponent].cfcSign[0].second = isImposed2D[noComponent][1];
			}
		} 
	}



	// ETAPE 6 DE LA PROCEDURE A DEUX DIMENSION 
	// -------------------------------------

	myGraphType cfcGraph;

	reduceGraph(fixedGraph, cfcGraph, nbComponents);

	// ETAPE 7 DE LA PROCEDURE A DEUX DIMENSION 
	// ----------------------------------------

	std::deque<int> topoOrder;

	if(boost::num_vertices(cfcGraph) > 1) {

		// Perform a topological sort on the graph whose vertices are the components (reduced graph)
		// -----------------------------------------------------------------------------------------

		boost::topological_sort(cfcGraph, std::front_inserter(topoOrder), boost::vertex_index_map(boost::identity_property_map()));

		// ETAPE 8 DE LA PROCEDURE A DEUX DIMENSION 
		// ----------------------------------------

		// Sort the middle structure table elements (2D graph components) accordignly to the sorting performed on the reduced graph 
		// ------------------------------------------------------------------------------------------------------------------------

		vector<cfcStruct> sortedCfcStructTable;

		for(int noComponent = 0; noComponent < topoOrder.size(); noComponent++) sortedCfcStructTable.push_back(middleComp[noComp][fixedDim][topoOrder[noComponent]]);

		middleComp[noComp][fixedDim].clear();

		for(int noComponent = 0; noComponent < topoOrder.size(); noComponent++) middleComp[noComp][fixedDim].push_back(sortedCfcStructTable[noComponent]);

	}

	else { topoOrder[0] = 0; }


	// ETAPE 9 DE LA PROCEDURE A DEUX DIMENSION 
	// ----------------------------------------

	// Create a graph that just keep the current 2D component egdes and vertices 
	// -------------------------------------------------------------------------

	for(int noComp2D = 0; noComp2D < nbComponents; noComp2D++) { 

		myGraphType outputGraph;

		copyGraph(fixedGraph, outputGraph); 

		keepComponent(topoOrder[noComp2D], outputGraph);


		// Create the two 1D component vectors corresponding to the two third dimensions availables
		// ----------------------------------------------------------------------------------------

		switch(fixedDim) {

			case 0:

				setInner(noComp, fixedDim, noComp2D, 1, 2, outputGraph);    // jk    
	
				setInner(noComp, fixedDim, noComp2D, 2, 1, outputGraph);    // kj

			break;

			case 1:

				setInner(noComp, fixedDim, noComp2D, 0, 2, outputGraph);    // ik

				setInner(noComp, fixedDim, noComp2D, 2, 0, outputGraph);    // ki

			break;

			case 2:

				setInner(noComp, fixedDim, noComp2D, 0, 1, outputGraph);    // ij

				setInner(noComp, fixedDim, noComp2D, 1, 0, outputGraph);    // ji

			break;
		}
	}
};



void Order::read1D() {

        // For all the components embedded in the 3D graph
        // -----------------------------------------------

        for(int noComp3D = 0; noComp3D < outerComp.size(); noComp3D++) {

                // For all the components embedded in the 2D graph coming from the current 3D component
                // ------------------------------------------------------------------------------------

                for(int noComp2D = 0; noComp2D < middleComp[noComp3D][0].size(); noComp2D++)  {

                        // For all the components embedded in the 1D graph coming from the current 2D component
                        // ------------------------------------------------------------------------------------

                        for(int noComp1D = 0; noComp1D < innerComp[noComp3D][0][noComp2D][1].size(); noComp1D++)  {

                                // Getting the value of the third axis (no need to get the number of the axis as there is only one)
                                // ------------------------------------------------------------------------------------------------     

                                int valAxis1 = innerComp[noComp3D][0][noComp2D][1][noComp1D].cfcSign[0].second;

                                // Read the order paramater for this axis value (axis number and parameter number are fixed as there is just one dimension)
                                // ------------------------------------------------------------------------------------------------------------------------

                                readParam(0, valAxis1, 1);

                                cout << endl << endl << "\t\t";

                                // For all the modules embedded in the current 1D component
                                // --------------------------------------------------------

                                for(int noMod = 0; noMod < innerComp[noComp3D][0][noComp2D][1][noComp1D].embeddedModuls.size(); noMod++) {

                                        // Insert the modules after the paramaters
                                        // ---------------------------------------

                                        cout << boost::get(boost::vertex_name_t(), myGraph, innerComp[noComp3D][0][noComp2D][1][noComp1D].embeddedModuls[noMod]) << " ";

                                }
                        }
                }
        }

        // Close the order statement
        // -------------------------

        readParam(0, 0, 0);

};



void Order::read2D() {


        // For all the components embedded in the 3D graph
        // -----------------------------------------------

        for(int noComp3D = 0; noComp3D < outerComp.size(); noComp3D++) {

                // For all the components embedded in the 2D graph coming from the current 3D component
                // ------------------------------------------------------------------------------------

                for(int noComp2D = 0; noComp2D < middleComp[noComp3D][0].size(); noComp2D++)  {

                        // Getting the axis number of the parameter 
                        // ----------------------------------------

                        int noFixed1  = middleComp[noComp3D][0][noComp2D].cfcSign[0].first;

                        // Getting the axis value of the parameter
                        // ---------------------------------------

                        int valFixed1 = middleComp[noComp3D][0][noComp2D].cfcSign[0].second;

                        // Read the order paramater for this axis number and this axis value (axis number is decreased as there is just two dimensions : the second axis become the first one)
                        // -------------------------------------------------------------------------------------------------------------------------------------------------------------------

                        readParam(noFixed1 -1, valFixed1, 1);

                        // Getting the axis number of the remaining parameter 
                        // --------------------------------------------------

                        int noAxis2  = middleComp[noComp3D][0][noComp2D].cfcSign[1].first;

                        // Getting the axis value of the remaining parameter 
                        // -------------------------------------------------

                        int valAxis2 = middleComp[noComp3D][0][noComp2D].cfcSign[1].second;

                        // If the third axis value is defined inside the current 2D component
                        // ------------------------------------------------------------------

                        if(valAxis2 != 2) {

                                // Read the order paramater for this axis number and this axis value (axis number is decreased as there is just two dimensions : the third axis become the second one)
                                // -------------------------------------------------------------------------------------------------------------------------------------------------------------------

                                readParam(noAxis2 -1, valAxis2, 2);

                                cout << endl << endl << "\t\t";

                                // For all the components embedded in the 1D graph coming from the current 2D component

                                for(int noComp1D = 0; noComp1D < innerComp[noComp3D][0][noComp2D][noAxis2].size(); noComp1D++) {

                                        // For all the modules embedded in the current 1D component
                                        // --------------------------------------------------------

                                        for(int noMod = 0; noMod < innerComp[noComp3D][0][noComp2D][noAxis2][noComp1D].embeddedModuls.size(); noMod++) {

                                                // Insert the moduls after the parameters
                                                // --------------------------------------

                                                cout << boost::get(boost::vertex_name_t(), myGraph, innerComp[noComp3D][0][noComp2D][noAxis2][noComp1D].embeddedModuls[noMod]) << " ";

                                        }
                                }
                        }

                        // If the third axis value is not defined inside the current 2D component
                        // ----------------------------------------------------------------------

                        else {

                                // For all the components embedded in the 1D graph coming from the current 2D component
                                // ------------------------------------------------------------------------------------

                                for(int noComp1D = 0; noComp1D < innerComp[noComp3D][0][noComp2D][noAxis2].size(); noComp1D++) {

                                        // Getting the axis number of the remaining parameter 
                                        // --------------------------------------------------

                                        noAxis2  = innerComp[noComp3D][0][noComp2D][noAxis2][noComp1D].cfcSign[0].first;

                                        // Getting the axis value of the remaining parameter 
                                        // -------------------------------------------------

                                        valAxis2 = innerComp[noComp3D][0][noComp2D][noAxis2][noComp1D].cfcSign[0].second;

                                        // Read the order paramater for this axis number and this axis value (axis number is decreased as there is just two dimensions : the third axis become the second one)
                                        // -------------------------------------------------------------------------------------------------------------------------------------------------------------------

                                        readParam(noAxis2 -1, valAxis2, 2);

                                        cout << endl << endl << "\t\t";

                                        // For all the modules embedded in the current 1D component
                                        // --------------------------------------------------------

                                        for(int noMod = 0; noMod < innerComp[noComp3D][0][noComp2D][noAxis2][noComp1D].embeddedModuls.size(); noMod++) {

                                                // Insert the moduls after the parameter
                                                // -------------------------------------
                                                                                                     
                                                cout << boost::get(boost::vertex_name_t(), myGraph, innerComp[noComp3D][0][noComp2D][noAxis2][noComp1D].embeddedModuls[noMod]) << " ";

                                        }
                                }
                        }
                }
        }

        // Close the order statement
        // -------------------------

        readParam(0, 0, 0);

};





void Order::read3D() {


	// For all the components embedded in the 3D graph
        // -----------------------------------------------

        for(int noComp3D = 0; noComp3D < outerComp.size(); noComp3D++) {

        // Get the axis number (dimension) and the axis value of the first axis crossed in the order statement
        // ---------------------------------------------------------------------------------------------------

        int noFixed1 = outerComp[noComp3D].cfcSign[0].first; int valFixed1 = outerComp[noComp3D].cfcSign[0].second;

        // Get the axis number (dimension) and the axis value of the second axis crossed in the order statement (values with no fixed dimensions : ie 3D graphs)
        // -----------------------------------------------------------------------------------------------------------------------------------------------------

        int noAxis2  = outerComp[noComp3D].cfcSign[1].first; int valAxis2  = outerComp[noComp3D].cfcSign[1].second;

        // Get the axis number (dimension) and the axis value of the third axis crossed in the order statement (values with no fixed dimensions : ie 3D graphs)
        // ----------------------------------------------------------------------------------------------------------------------------------------------------

        int noAxis3 = outerComp[noComp3D].cfcSign[2].first; int valAxis3   = outerComp[noComp3D].cfcSign[2].second;

        // Read the parameter for the first axis (this one can not be undefined as it is the first axis crossed, the the reading can be performed) 
        // ---------------------------------------------------------------------------------------------------------------------------------------

        readParam(noFixed1, valFixed1, 1);

        // If the second axis value is defined in the 3D graph 
        // ---------------------------------------------------

        if(valAxis2 != 2) {

                // Read the parameter for the second axis directly from here (no need to get informations from any fixed graph)
                // ------------------------------------------------------------------------------------------------------------

                 readParam(noAxis2, valAxis2, 2);

                // If the third axis value is defined in the 3D graph
                // --------------------------------------------------

                if(valAxis3 != 2) {

                                // Read the parameter for the third axis directly from here (no need to get informations from any fixed graph)
                                // -----------------------------------------------------------------------------------------------------------

                                readParam(noAxis3, valAxis3, 3);

                                cout << endl << endl << "\t\t\t";

                                // For all 2D graphs embedded in the current 3D graph (ie the fixed dimension is the first crossed in the order statement)
                                // ----------------------------------------------------------------------------------------------------------------------- 

                                for(int noComp2D = 0; noComp2D < middleComp[noComp3D][noFixed1].size(); noComp2D++) {

                                        // Get the number of the second axis crossed in the order statement 
                                        // ----------------------------------------------------------------

                                        noAxis2 = middleComp[noComp3D][noFixed1][noComp2D].cfcSign[0].first;

                                        // For all 1D graphs embedded in the current 2D graph (ie the second dimension is the second crossed in the order statement)
                                        // -------------------------------------------------------------------------------------------------------------------------    

                                        for(int noComp1D= 0; noComp1D < innerComp[noComp3D][noFixed1][noComp2D][noAxis2].size(); noComp1D++) {

                                                // For all moduls embedded in the current 1D graph component
                                                // ---------------------------------------------------------

                                                for(int noMod = 0; noMod < innerComp[noComp3D][noFixed1][noComp2D][noAxis2][noComp1D].embeddedModuls.size(); noMod++) {

                                                        // Display the modul name (that will appear after the order paramaters that have been read before)
                                                        // -----------------------------------------------------------------------------------------------      

                                                        cout << boost::get(boost::vertex_name_t(), myGraph, innerComp[noComp3D][noFixed1][noComp2D][noAxis2][noComp1D].embeddedModuls[noMod]) << " ";
                                                }
                                        }
                                }
                        }

                // If the third axis value is not defined in the 2D graph 

                else {

                        // For all 2D graphs embedded in the current 3D graph (ie the fixed dimension is the first crossed in the order statement)
                        // -----------------------------------------------------------------------------------------------------------------------

                        for(int noComp2D = 0; noComp2D < middleComp[noComp3D][noFixed1].size(); noComp2D++) {


                                // Get the number of the second axis crossed in the order statement
                                // ----------------------------------------------------------------

                                noAxis2 = middleComp[noComp3D][noFixed1][noComp2D].cfcSign[0].first;

                                // For all 1D graphs embedded in the current 2D graph (ie the second dimension is the second crossed in the order statement)
                                // -------------------------------------------------------------------------------------------------------------------------

                                for(int noComp1D = 0; noComp1D < innerComp[noComp3D][noFixed1][noComp2D][noAxis2].size(); noComp1D++) {

                                        // Get the number of the third axis crossed in the order statement
                                        // ---------------------------------------------------------------

                                        noAxis3  = innerComp[noComp3D][noFixed1][noComp2D][noAxis2][noComp1D].cfcSign[0].first;

                                        // Get the value of the third axis crossed in the order statement
                                        // --------------------------------------------------------------

                                        valAxis3 = innerComp[noComp3D][noFixed1][noComp2D][noAxis2][noComp1D].cfcSign[0].second;

                                        // Read the parameter for the third axis from the 1D component
                                        // -----------------------------------------------------------

                                        readParam(noAxis3, valAxis3, 3);

                                        cout << endl << endl << "\t\t\t";

                                        // For all moduls embedded in the current 1D graph component
                                        // ---------------------------------------------------------

                                        for(int noMod = 0; noMod < innerComp[noComp3D][noFixed1][noComp2D][noAxis2][noComp1D].embeddedModuls.size(); noMod++) {

                                                // Display the modul name (that will appear after the order paramaters that have been read before)
                                                // -----------------------------------------------------------------------------------------------

                                                cout << boost::get(boost::vertex_name_t(), myGraph, innerComp[noComp3D][noFixed1][noComp2D][noAxis2][noComp1D].embeddedModuls[noMod]) << " ";

                                        }
                             }
                       }
                }


        }

        // If the second axis value is not defined in the 3D graph 
        // -------------------------------------------------------

        if(valAxis2 == 2) {

                // For all 2D graphs embedded in the current 3D graph (ie the fixed dimension is the first crossed in the order statement)
                // -----------------------------------------------------------------------------------------------------------------------

                for(int noComp2D = 0; noComp2D < middleComp[noComp3D][noFixed1].size(); noComp2D++) {

                        // Get the number of the second axis crossed in the order statement
                        // ----------------------------------------------------------------

                        noAxis2  = middleComp[noComp3D][noFixed1][noComp2D].cfcSign[0].first;

                        // Get the value of the second axis crossed in the order statement
                        // ----------------------------------------------------------------

                        valAxis2 = middleComp[noComp3D][noFixed1][noComp2D].cfcSign[0].second;

                        // Get the number of the third axis crossed in the order statement (values with just one fixed dimensions : ie from 2D graphs)
                        // ---------------------------------------------------------------------------------------------------------------------------

                        noAxis3  = middleComp[noComp3D][noFixed1][noComp2D].cfcSign[1].first;

                        // Get the value of the third axis crossed in the order statement (values with just one fixed dimensions : ie from 2D graphs)
                        // --------------------------------------------------------------------------------------------------------------------------

                        valAxis3 = middleComp[noComp3D][noFixed1][noComp2D].cfcSign[1].second;

                        // If the second axis value is defined in the 2D graph
                        // ---------------------------------------------------

                        if(valAxis2 != 2) {

                                // Read the parameter for the second axis from the 2D component
                                // ------------------------------------------------------------

                                readParam(noAxis2, valAxis2, 2);

                                // If the third axis value is defined in the 2D graph
                                // --------------------------------------------------

                                if(valAxis3 != 2) {

                                        // Read the parameter for the third axis from the 2D component
                                        // -----------------------------------------------------------

                                        readParam(noAxis3, valAxis3, 3);

                                        cout << endl << endl << "\t\t\t";

                                        // For all 1D graph embedded in the current 2D graph (ie the second fixed dimension is the second crossed in the order statement)
                                        // ------------------------------------------------------------------------------------------------------------------------------

                                        for(int noComp1D= 0; noComp1D < innerComp[noComp3D][noFixed1][noComp2D][noAxis2].size(); noComp1D++) {

                                                // For all moduls embedded in the current 1D graph component
                                                // ---------------------------------------------------------

                                                for(int noMod = 0; noMod < innerComp[noComp3D][noFixed1][noComp2D][noAxis2][noComp1D].embeddedModuls.size(); noMod++) {

                                                        // Display the modul name (that will appear after the order paramaters that have been readen before)
                                                        // -------------------------------------------------------------------------------------------------

                                                        cout << boost::get(boost::vertex_name_t(), myGraph, innerComp[noComp3D][noFixed1][noComp2D][noAxis2][noComp1D].embeddedModuls[noMod]) << " ";

                                                }
                                        }
                                }

                                // If the third axis value is not defined in the 2D graph
                                // ------------------------------------------------------

                                else {

                                        // For all 1D graph embedded in the current 2D graph (ie the second fixed dimension is the second crossed in the order statement)
                                        // ------------------------------------------------------------------------------------------------------------------------------

                                        for(int noComp1D= 0; noComp1D < innerComp[noComp3D][noFixed1][noComp2D][noAxis2].size(); noComp1D++) {

                                                // Get the number of the third axis crossed in the order statement
                                                // ---------------------------------------------------------------

                                                noAxis3  = innerComp[noComp3D][noFixed1][noComp2D][noAxis2][noComp1D].cfcSign[0].first;

                                                // Get the value of the third axis crossed in the order statement
                                                // ---------------------------------------------------------------

                                                valAxis3 = innerComp[noComp3D][noFixed1][noComp2D][noAxis2][noComp1D].cfcSign[0].second;

                                                // Read the parameter for the third axis from the 1D component
                                                // -----------------------------------------------------------

                                                readParam(noAxis3, valAxis3, 3);

                                                cout << endl << endl << "\t\t\t";

                                                // For all moduls embedded in the current 1D graph component
                                                // ---------------------------------------------------------

                                                for(int noMod = 0; noMod < innerComp[noComp3D][noFixed1][noComp2D][noAxis2][noComp1D].embeddedModuls.size(); noMod++) {

                                                        // Display the modul name (that will appear after the order paramaters that have been readen before)
                                                        // -------------------------------------------------------------------------------------------------

                                                        cout << boost::get(boost::vertex_name_t(), myGraph, innerComp[noComp3D][noFixed1][noComp2D][noAxis2][noComp1D].embeddedModuls[noMod]) << " ";

                                                }
                                        }
                                }
                        }
                }
        }

	}

        // Close the order statement
        // -------------------------

        readParam(0, 0, 0);
};





int Order::getImposed(int fixedDim, int freeDim, int noComp, myGraphType& upperGraph) {

        // Create a graph that will be fixed
        // ---------------------------------

        myGraphType fixedGraph;

        // Copy the informations of the graph given as parameter in the graph that will be fixed
        // -------------------------------------------------------------------------------------

        copyGraph(upperGraph, fixedGraph);

        // Fix the dimension "fixedDim" given as parameter
        // -----------------------------------------------

        fixGraph(fixedGraph, fixedDim);

        // If there is just one vertex without reflexive edge in the component, then hide the returned value by setting it as free (ie 0 instead of 3)
        // -------------------------------------------------------------------------------------------------------------------------------------------

	int value = getCommonSign(freeDim, noComp, fixedGraph);

	if(value == 3) value = 0;

	// Return the sign for the free dimension and the component given as parameter 
        // ---------------------------------------------------------------------------

        return(value);

};




bool Order::isToDecompose(int freeDim, myGraphType& upperGraph) {


        // A boolean value that will be set to true if the dimension have to be decomposed 
        // -------------------------------------------------------------------------------

        bool toDecompose = false;

        // Create a graph that will be fixed
        // ---------------------------------

        myGraphType fixedGraph;

        // Copy the informations of the graph given as parameter in the graph that will be fixed
        // -------------------------------------------------------------------------------------

        copyGraph(upperGraph, fixedGraph);

        // Accordingly to the dimension sign that must be analyzed (free dimension), fix both of the two other dimensions
        // --------------------------------------------------------------------------------------------------------------

        if(freeDim == 0) { fixGraph(fixedGraph, 1); fixGraph(fixedGraph, 2); }  // j == 0, k == 0
        if(freeDim == 1) { fixGraph(fixedGraph, 0); fixGraph(fixedGraph, 2); }  // i == 0, k == 0
        if(freeDim == 2) { fixGraph(fixedGraph, 0); fixGraph(fixedGraph, 1); }  // i == 0, j == 0

        // If the sign of the remaining edges is not defined for the free dimension given as parameter, then the dimension have to be decomposed
        // -------------------------------------------------------------------------------------------------------------------------------------

        if(getCommonSign(freeDim, fixedGraph) == 2) toDecompose = true;

        // Return the boolean information about decomposition of the dimension 
        // -------------------------------------------------------------------

        return(toDecompose);

};



int Order::getCommonSign(int noDim, int noComp, myGraphType& currentGraph) {

        int valDim0 = 0;

	// A value that will set to true only if it exist an edge between the component vertices
	// -------------------------------------------------------------------------------------

	bool isConnected = false; 
	
        // For all edges in the current graph
        // ----------------------------------

        for(tie(edgeIterOne, edgeIterTwo) = edges(currentGraph); edgeIterOne != edgeIterTwo; ++edgeIterOne) {

                // If the source modul of the edge is in the component given as paramater
                // ----------------------------------------------------------------------

                if(get(boost::vertex_affected_comp_t(), currentGraph, boost::source(*edgeIterOne, currentGraph)) == noComp)  {

                        // If the target modul of the edge is in the component given as paramater
                        // ----------------------------------------------------------------------

                        if(get(boost::vertex_affected_comp_t(), currentGraph, boost::target(*edgeIterOne, currentGraph)) == noComp)  {

				// Update isConnected as it exist a connexion inside the component
				// ---------------------------------------------------------------

				isConnected = true;

                                // Get the dimension values for axis i, j and k 
                                // --------------------------------------------

                                int val0 = get(boost::edge_weight_d1_t(), currentGraph, *edgeIterOne);
                                int val1 = get(boost::edge_weight_d2_t(), currentGraph, *edgeIterOne);
                                int val2 = get(boost::edge_weight_d3_t(), currentGraph, *edgeIterOne);

                                // If the recorded value valDim0 is not equal to 2 (ie. if it is equal to 2, there is no need to go further in the sign analyzis)
                                // -----------------------------------------------------------------------------------------------------------------------------

                                if(valDim0 != 2) {

                                        // The sign analyzis is performed for the dimension "noDim" given as parameter 
                                        // ---------------------------------------------------------------------------

                                        switch(noDim) {

                                                // If the current value is not null and valDim0 + the current value is equal to zero, it means there is a couple -1 an +1 and then no common sign
                                                // ----------------------------------------------------------------------------------------------------------------------------------------------

                                                case 0: if(val0 != 0) { if(valDim0 + val0 == 0) { valDim0 = 2; } else { valDim0  = val0; } } break;
                                                case 1: if(val1 != 0) { if(valDim0 + val1 == 0) { valDim0 = 2; } else { valDim0  = val1; } } break;
                                                case 2: if(val2 != 0) { if(valDim0 + val2 == 0) { valDim0 = 2; } else { valDim0  = val2; } } break;
                                        }
                                }
                        }
                }
        }

        if(isConnected) return(valDim0); 
	return(3); 

};




int Order::getImposed(int freeDim, int noComp, myGraphType& upperGraph) {

        // Create a new graph that will be fixed

        myGraphType fixedGraph;

        // Copy the informations of the graph given as parameter in the graph that will be fixed
        // -------------------------------------------------------------------------------------

        copyGraph(upperGraph, fixedGraph);

        // Accordingly to the dimension sign that must be analyzed (free dimension), fix both of the two other dimensions
        // --------------------------------------------------------------------------------------------------------------

        if(freeDim == 0) { fixGraph(fixedGraph, 1); fixGraph(fixedGraph, 2); }  // j == 0, k == 0
        if(freeDim == 1) { fixGraph(fixedGraph, 0); fixGraph(fixedGraph, 2); }  // i == 0, k == 0
        if(freeDim == 2) { fixGraph(fixedGraph, 0); fixGraph(fixedGraph, 1); }  // i == 0, j == 0

	// If there is just one vertex without reflexive edge in the component, then hide the returned value by setting it as free (ie 0 instead of 3)
        // -------------------------------------------------------------------------------------------------------------------------------------------

        int value = getCommonSign(freeDim, noComp, fixedGraph);

        if(value == 3) value = 0;

        // Return the sign for the free dimension and the component given as parameter 
        // ---------------------------------------------------------------------------

	return(value);

};
	

void Order::resetGraph(myGraphType& currentGraph) {

        // Putting iterators inside a vector is more convenient , especially for deletions                                                                             
        // -------------------------------------------------------------------------------

        typedef boost::graph_traits<myGraphType>::edge_iterator edgeIterType;

        std::vector<edgeIterType> edgeIterVector;

        // Insert all edges of the graph inside a vector of edge
        // -----------------------------------------------------   

        for(tie(edgeIterOne, edgeIterTwo) = edges(currentGraph); edgeIterOne != edgeIterTwo; ++edgeIterOne) edgeIterVector.push_back(edgeIterOne);

        // Reverse the vector to avoid a mess in the indexes when the edges are deleted
        // ----------------------------------------------------------------------------                                      

        reverse(edgeIterVector.begin(), edgeIterVector.end());

        // For all edges inside the vector
        // -------------------------------                                                                        

        for(int i = 0; i < edgeIterVector.size(); i++) boost::remove_edge(*(edgeIterVector[i]), currentGraph);

        // Putting iterators inside a vector is more convenient , especially for deletions                                                                             
        // -------------------------------------------------------------------------------

        typedef boost::graph_traits<myGraphType>::vertex_iterator vertexIterType;

        std::vector<vertexIterType> vertexIterVector;

        // Insert all vertices of the graph inside a vector of vertex
        // ----------------------------------------------------------

        for(tie(vertexIterOne, vertexIterTwo) = vertices(currentGraph); vertexIterOne != vertexIterTwo; ++vertexIterOne) vertexIterVector.push_back(vertexIterOne);

        // Reverse the vector to avoid a mess in the indexes when the vertices are deleted
        // ----------------------------------------------------------------------------

        reverse(vertexIterVector.begin(), vertexIterVector.end());

        // For all vertices embeded in the vector
        // -----------------------------------

        for(int i = 0; i < vertexIterVector.size(); i++) boost::remove_vertex(*(vertexIterVector[i]), currentGraph);


        } ;




//-----------------CB---------------------

